"""Helper functions for working with Angzarr proto types."""

from datetime import datetime
from typing import Optional, Union
from uuid import UUID as PyUUID

from google.protobuf.timestamp_pb2 import Timestamp
from google.protobuf.any_pb2 import Any as ProtoAny

from .proto.angzarr import (
    UUID,
    Cover,
    Edition,
    DomainDivergence,
    EventBook,
    CommandBook,
    Query,
    EventPage,
    CommandPage,
    SequenceRange,
    TemporalQuery,
)
from .errors import InvalidTimestampError

# Constants matching Rust proto_ext::constants
UNKNOWN_DOMAIN = "unknown"
WILDCARD_DOMAIN = "*"
DEFAULT_EDITION = "angzarr"
META_ANGZARR_DOMAIN = "_angzarr"
PROJECTION_DOMAIN_PREFIX = "projection:"
CORRELATION_ID_HEADER = "x-correlation-id"
TYPE_URL_PREFIX = "type.googleapis.com/"


# Type for Cover-bearing objects
CoverBearer = Union[EventBook, CommandBook, Query, Cover]
from inspect import signature as _mutmut_signature
from typing import Annotated
from typing import Callable
from typing import ClassVar


MutantDict = Annotated[dict[str, Callable], "Mutant"]


def _mutmut_trampoline(orig, mutants, call_args, call_kwargs, self_arg = None):
    """Forward call to original or mutated function, depending on the environment"""
    import os
    mutant_under_test = os.environ['MUTANT_UNDER_TEST']
    if mutant_under_test == 'fail':
        from mutmut.__main__ import MutmutProgrammaticFailException
        raise MutmutProgrammaticFailException('Failed programmatically')      
    elif mutant_under_test == 'stats':
        from mutmut.__main__ import record_trampoline_hit
        record_trampoline_hit(orig.__module__ + '.' + orig.__name__)
        result = orig(*call_args, **call_kwargs)
        return result
    prefix = orig.__module__ + '.' + orig.__name__ + '__mutmut_'
    if not mutant_under_test.startswith(prefix):
        result = orig(*call_args, **call_kwargs)
        return result
    mutant_name = mutant_under_test.rpartition('.')[-1]
    if self_arg is not None:
        # call to a class method where self is not bound
        result = mutants[mutant_name](self_arg, *call_args, **call_kwargs)
    else:
        result = mutants[mutant_name](*call_args, **call_kwargs)
    return result


def x_cover_of__mutmut_orig(obj: CoverBearer) -> Optional[Cover]:
    """Extract the Cover from various proto types."""
    if isinstance(obj, Cover):
        return obj
    if hasattr(obj, "cover"):
        return obj.cover
    return None


def x_cover_of__mutmut_1(obj: CoverBearer) -> Optional[Cover]:
    """Extract the Cover from various proto types."""
    if isinstance(obj, Cover):
        return obj
    if hasattr(None, "cover"):
        return obj.cover
    return None


def x_cover_of__mutmut_2(obj: CoverBearer) -> Optional[Cover]:
    """Extract the Cover from various proto types."""
    if isinstance(obj, Cover):
        return obj
    if hasattr(obj, None):
        return obj.cover
    return None


def x_cover_of__mutmut_3(obj: CoverBearer) -> Optional[Cover]:
    """Extract the Cover from various proto types."""
    if isinstance(obj, Cover):
        return obj
    if hasattr("cover"):
        return obj.cover
    return None


def x_cover_of__mutmut_4(obj: CoverBearer) -> Optional[Cover]:
    """Extract the Cover from various proto types."""
    if isinstance(obj, Cover):
        return obj
    if hasattr(obj, ):
        return obj.cover
    return None


def x_cover_of__mutmut_5(obj: CoverBearer) -> Optional[Cover]:
    """Extract the Cover from various proto types."""
    if isinstance(obj, Cover):
        return obj
    if hasattr(obj, "XXcoverXX"):
        return obj.cover
    return None


def x_cover_of__mutmut_6(obj: CoverBearer) -> Optional[Cover]:
    """Extract the Cover from various proto types."""
    if isinstance(obj, Cover):
        return obj
    if hasattr(obj, "COVER"):
        return obj.cover
    return None

x_cover_of__mutmut_mutants : ClassVar[MutantDict] = {
'x_cover_of__mutmut_1': x_cover_of__mutmut_1, 
    'x_cover_of__mutmut_2': x_cover_of__mutmut_2, 
    'x_cover_of__mutmut_3': x_cover_of__mutmut_3, 
    'x_cover_of__mutmut_4': x_cover_of__mutmut_4, 
    'x_cover_of__mutmut_5': x_cover_of__mutmut_5, 
    'x_cover_of__mutmut_6': x_cover_of__mutmut_6
}

def cover_of(*args, **kwargs):
    result = _mutmut_trampoline(x_cover_of__mutmut_orig, x_cover_of__mutmut_mutants, args, kwargs)
    return result 

cover_of.__signature__ = _mutmut_signature(x_cover_of__mutmut_orig)
x_cover_of__mutmut_orig.__name__ = 'x_cover_of'


def x_domain__mutmut_orig(obj: CoverBearer) -> str:
    """Get the domain from a Cover-bearing type, or UNKNOWN_DOMAIN if missing."""
    c = cover_of(obj)
    if c is None or not c.domain:
        return UNKNOWN_DOMAIN
    return c.domain


def x_domain__mutmut_1(obj: CoverBearer) -> str:
    """Get the domain from a Cover-bearing type, or UNKNOWN_DOMAIN if missing."""
    c = None
    if c is None or not c.domain:
        return UNKNOWN_DOMAIN
    return c.domain


def x_domain__mutmut_2(obj: CoverBearer) -> str:
    """Get the domain from a Cover-bearing type, or UNKNOWN_DOMAIN if missing."""
    c = cover_of(None)
    if c is None or not c.domain:
        return UNKNOWN_DOMAIN
    return c.domain


def x_domain__mutmut_3(obj: CoverBearer) -> str:
    """Get the domain from a Cover-bearing type, or UNKNOWN_DOMAIN if missing."""
    c = cover_of(obj)
    if c is None and not c.domain:
        return UNKNOWN_DOMAIN
    return c.domain


def x_domain__mutmut_4(obj: CoverBearer) -> str:
    """Get the domain from a Cover-bearing type, or UNKNOWN_DOMAIN if missing."""
    c = cover_of(obj)
    if c is not None or not c.domain:
        return UNKNOWN_DOMAIN
    return c.domain


def x_domain__mutmut_5(obj: CoverBearer) -> str:
    """Get the domain from a Cover-bearing type, or UNKNOWN_DOMAIN if missing."""
    c = cover_of(obj)
    if c is None or c.domain:
        return UNKNOWN_DOMAIN
    return c.domain

x_domain__mutmut_mutants : ClassVar[MutantDict] = {
'x_domain__mutmut_1': x_domain__mutmut_1, 
    'x_domain__mutmut_2': x_domain__mutmut_2, 
    'x_domain__mutmut_3': x_domain__mutmut_3, 
    'x_domain__mutmut_4': x_domain__mutmut_4, 
    'x_domain__mutmut_5': x_domain__mutmut_5
}

def domain(*args, **kwargs):
    result = _mutmut_trampoline(x_domain__mutmut_orig, x_domain__mutmut_mutants, args, kwargs)
    return result 

domain.__signature__ = _mutmut_signature(x_domain__mutmut_orig)
x_domain__mutmut_orig.__name__ = 'x_domain'


def x_correlation_id__mutmut_orig(obj: CoverBearer) -> str:
    """Get the correlation_id from a Cover-bearing type, or empty string if missing."""
    c = cover_of(obj)
    if c is None:
        return ""
    return c.correlation_id


def x_correlation_id__mutmut_1(obj: CoverBearer) -> str:
    """Get the correlation_id from a Cover-bearing type, or empty string if missing."""
    c = None
    if c is None:
        return ""
    return c.correlation_id


def x_correlation_id__mutmut_2(obj: CoverBearer) -> str:
    """Get the correlation_id from a Cover-bearing type, or empty string if missing."""
    c = cover_of(None)
    if c is None:
        return ""
    return c.correlation_id


def x_correlation_id__mutmut_3(obj: CoverBearer) -> str:
    """Get the correlation_id from a Cover-bearing type, or empty string if missing."""
    c = cover_of(obj)
    if c is not None:
        return ""
    return c.correlation_id


def x_correlation_id__mutmut_4(obj: CoverBearer) -> str:
    """Get the correlation_id from a Cover-bearing type, or empty string if missing."""
    c = cover_of(obj)
    if c is None:
        return "XXXX"
    return c.correlation_id

x_correlation_id__mutmut_mutants : ClassVar[MutantDict] = {
'x_correlation_id__mutmut_1': x_correlation_id__mutmut_1, 
    'x_correlation_id__mutmut_2': x_correlation_id__mutmut_2, 
    'x_correlation_id__mutmut_3': x_correlation_id__mutmut_3, 
    'x_correlation_id__mutmut_4': x_correlation_id__mutmut_4
}

def correlation_id(*args, **kwargs):
    result = _mutmut_trampoline(x_correlation_id__mutmut_orig, x_correlation_id__mutmut_mutants, args, kwargs)
    return result 

correlation_id.__signature__ = _mutmut_signature(x_correlation_id__mutmut_orig)
x_correlation_id__mutmut_orig.__name__ = 'x_correlation_id'


def x_has_correlation_id__mutmut_orig(obj: CoverBearer) -> bool:
    """Return True if the correlation_id is present and non-empty."""
    return bool(correlation_id(obj))


def x_has_correlation_id__mutmut_1(obj: CoverBearer) -> bool:
    """Return True if the correlation_id is present and non-empty."""
    return bool(None)


def x_has_correlation_id__mutmut_2(obj: CoverBearer) -> bool:
    """Return True if the correlation_id is present and non-empty."""
    return bool(correlation_id(None))

x_has_correlation_id__mutmut_mutants : ClassVar[MutantDict] = {
'x_has_correlation_id__mutmut_1': x_has_correlation_id__mutmut_1, 
    'x_has_correlation_id__mutmut_2': x_has_correlation_id__mutmut_2
}

def has_correlation_id(*args, **kwargs):
    result = _mutmut_trampoline(x_has_correlation_id__mutmut_orig, x_has_correlation_id__mutmut_mutants, args, kwargs)
    return result 

has_correlation_id.__signature__ = _mutmut_signature(x_has_correlation_id__mutmut_orig)
x_has_correlation_id__mutmut_orig.__name__ = 'x_has_correlation_id'


def x_root_uuid__mutmut_orig(obj: CoverBearer) -> Optional[PyUUID]:
    """Extract the root UUID from a Cover-bearing type."""
    c = cover_of(obj)
    if c is None or not c.HasField("root"):
        return None
    try:
        return PyUUID(bytes=c.root.value)
    except ValueError:
        return None


def x_root_uuid__mutmut_1(obj: CoverBearer) -> Optional[PyUUID]:
    """Extract the root UUID from a Cover-bearing type."""
    c = None
    if c is None or not c.HasField("root"):
        return None
    try:
        return PyUUID(bytes=c.root.value)
    except ValueError:
        return None


def x_root_uuid__mutmut_2(obj: CoverBearer) -> Optional[PyUUID]:
    """Extract the root UUID from a Cover-bearing type."""
    c = cover_of(None)
    if c is None or not c.HasField("root"):
        return None
    try:
        return PyUUID(bytes=c.root.value)
    except ValueError:
        return None


def x_root_uuid__mutmut_3(obj: CoverBearer) -> Optional[PyUUID]:
    """Extract the root UUID from a Cover-bearing type."""
    c = cover_of(obj)
    if c is None and not c.HasField("root"):
        return None
    try:
        return PyUUID(bytes=c.root.value)
    except ValueError:
        return None


def x_root_uuid__mutmut_4(obj: CoverBearer) -> Optional[PyUUID]:
    """Extract the root UUID from a Cover-bearing type."""
    c = cover_of(obj)
    if c is not None or not c.HasField("root"):
        return None
    try:
        return PyUUID(bytes=c.root.value)
    except ValueError:
        return None


def x_root_uuid__mutmut_5(obj: CoverBearer) -> Optional[PyUUID]:
    """Extract the root UUID from a Cover-bearing type."""
    c = cover_of(obj)
    if c is None or c.HasField("root"):
        return None
    try:
        return PyUUID(bytes=c.root.value)
    except ValueError:
        return None


def x_root_uuid__mutmut_6(obj: CoverBearer) -> Optional[PyUUID]:
    """Extract the root UUID from a Cover-bearing type."""
    c = cover_of(obj)
    if c is None or not c.HasField(None):
        return None
    try:
        return PyUUID(bytes=c.root.value)
    except ValueError:
        return None


def x_root_uuid__mutmut_7(obj: CoverBearer) -> Optional[PyUUID]:
    """Extract the root UUID from a Cover-bearing type."""
    c = cover_of(obj)
    if c is None or not c.HasField("XXrootXX"):
        return None
    try:
        return PyUUID(bytes=c.root.value)
    except ValueError:
        return None


def x_root_uuid__mutmut_8(obj: CoverBearer) -> Optional[PyUUID]:
    """Extract the root UUID from a Cover-bearing type."""
    c = cover_of(obj)
    if c is None or not c.HasField("ROOT"):
        return None
    try:
        return PyUUID(bytes=c.root.value)
    except ValueError:
        return None


def x_root_uuid__mutmut_9(obj: CoverBearer) -> Optional[PyUUID]:
    """Extract the root UUID from a Cover-bearing type."""
    c = cover_of(obj)
    if c is None or not c.HasField("root"):
        return None
    try:
        return PyUUID(bytes=None)
    except ValueError:
        return None

x_root_uuid__mutmut_mutants : ClassVar[MutantDict] = {
'x_root_uuid__mutmut_1': x_root_uuid__mutmut_1, 
    'x_root_uuid__mutmut_2': x_root_uuid__mutmut_2, 
    'x_root_uuid__mutmut_3': x_root_uuid__mutmut_3, 
    'x_root_uuid__mutmut_4': x_root_uuid__mutmut_4, 
    'x_root_uuid__mutmut_5': x_root_uuid__mutmut_5, 
    'x_root_uuid__mutmut_6': x_root_uuid__mutmut_6, 
    'x_root_uuid__mutmut_7': x_root_uuid__mutmut_7, 
    'x_root_uuid__mutmut_8': x_root_uuid__mutmut_8, 
    'x_root_uuid__mutmut_9': x_root_uuid__mutmut_9
}

def root_uuid(*args, **kwargs):
    result = _mutmut_trampoline(x_root_uuid__mutmut_orig, x_root_uuid__mutmut_mutants, args, kwargs)
    return result 

root_uuid.__signature__ = _mutmut_signature(x_root_uuid__mutmut_orig)
x_root_uuid__mutmut_orig.__name__ = 'x_root_uuid'


def x_root_id_hex__mutmut_orig(obj: CoverBearer) -> str:
    """Return the root UUID as a hex string, or empty string if missing."""
    c = cover_of(obj)
    if c is None or not c.HasField("root"):
        return ""
    return c.root.value.hex()


def x_root_id_hex__mutmut_1(obj: CoverBearer) -> str:
    """Return the root UUID as a hex string, or empty string if missing."""
    c = None
    if c is None or not c.HasField("root"):
        return ""
    return c.root.value.hex()


def x_root_id_hex__mutmut_2(obj: CoverBearer) -> str:
    """Return the root UUID as a hex string, or empty string if missing."""
    c = cover_of(None)
    if c is None or not c.HasField("root"):
        return ""
    return c.root.value.hex()


def x_root_id_hex__mutmut_3(obj: CoverBearer) -> str:
    """Return the root UUID as a hex string, or empty string if missing."""
    c = cover_of(obj)
    if c is None and not c.HasField("root"):
        return ""
    return c.root.value.hex()


def x_root_id_hex__mutmut_4(obj: CoverBearer) -> str:
    """Return the root UUID as a hex string, or empty string if missing."""
    c = cover_of(obj)
    if c is not None or not c.HasField("root"):
        return ""
    return c.root.value.hex()


def x_root_id_hex__mutmut_5(obj: CoverBearer) -> str:
    """Return the root UUID as a hex string, or empty string if missing."""
    c = cover_of(obj)
    if c is None or c.HasField("root"):
        return ""
    return c.root.value.hex()


def x_root_id_hex__mutmut_6(obj: CoverBearer) -> str:
    """Return the root UUID as a hex string, or empty string if missing."""
    c = cover_of(obj)
    if c is None or not c.HasField(None):
        return ""
    return c.root.value.hex()


def x_root_id_hex__mutmut_7(obj: CoverBearer) -> str:
    """Return the root UUID as a hex string, or empty string if missing."""
    c = cover_of(obj)
    if c is None or not c.HasField("XXrootXX"):
        return ""
    return c.root.value.hex()


def x_root_id_hex__mutmut_8(obj: CoverBearer) -> str:
    """Return the root UUID as a hex string, or empty string if missing."""
    c = cover_of(obj)
    if c is None or not c.HasField("ROOT"):
        return ""
    return c.root.value.hex()


def x_root_id_hex__mutmut_9(obj: CoverBearer) -> str:
    """Return the root UUID as a hex string, or empty string if missing."""
    c = cover_of(obj)
    if c is None or not c.HasField("root"):
        return "XXXX"
    return c.root.value.hex()

x_root_id_hex__mutmut_mutants : ClassVar[MutantDict] = {
'x_root_id_hex__mutmut_1': x_root_id_hex__mutmut_1, 
    'x_root_id_hex__mutmut_2': x_root_id_hex__mutmut_2, 
    'x_root_id_hex__mutmut_3': x_root_id_hex__mutmut_3, 
    'x_root_id_hex__mutmut_4': x_root_id_hex__mutmut_4, 
    'x_root_id_hex__mutmut_5': x_root_id_hex__mutmut_5, 
    'x_root_id_hex__mutmut_6': x_root_id_hex__mutmut_6, 
    'x_root_id_hex__mutmut_7': x_root_id_hex__mutmut_7, 
    'x_root_id_hex__mutmut_8': x_root_id_hex__mutmut_8, 
    'x_root_id_hex__mutmut_9': x_root_id_hex__mutmut_9
}

def root_id_hex(*args, **kwargs):
    result = _mutmut_trampoline(x_root_id_hex__mutmut_orig, x_root_id_hex__mutmut_mutants, args, kwargs)
    return result 

root_id_hex.__signature__ = _mutmut_signature(x_root_id_hex__mutmut_orig)
x_root_id_hex__mutmut_orig.__name__ = 'x_root_id_hex'


def x_edition__mutmut_orig(obj: CoverBearer) -> str:
    """Return the edition name from a Cover-bearing type, defaulting to DEFAULT_EDITION."""
    c = cover_of(obj)
    if c is None or not c.HasField("edition") or not c.edition.name:
        return DEFAULT_EDITION
    return c.edition.name


def x_edition__mutmut_1(obj: CoverBearer) -> str:
    """Return the edition name from a Cover-bearing type, defaulting to DEFAULT_EDITION."""
    c = None
    if c is None or not c.HasField("edition") or not c.edition.name:
        return DEFAULT_EDITION
    return c.edition.name


def x_edition__mutmut_2(obj: CoverBearer) -> str:
    """Return the edition name from a Cover-bearing type, defaulting to DEFAULT_EDITION."""
    c = cover_of(None)
    if c is None or not c.HasField("edition") or not c.edition.name:
        return DEFAULT_EDITION
    return c.edition.name


def x_edition__mutmut_3(obj: CoverBearer) -> str:
    """Return the edition name from a Cover-bearing type, defaulting to DEFAULT_EDITION."""
    c = cover_of(obj)
    if c is None or not c.HasField("edition") and not c.edition.name:
        return DEFAULT_EDITION
    return c.edition.name


def x_edition__mutmut_4(obj: CoverBearer) -> str:
    """Return the edition name from a Cover-bearing type, defaulting to DEFAULT_EDITION."""
    c = cover_of(obj)
    if c is None and not c.HasField("edition") or not c.edition.name:
        return DEFAULT_EDITION
    return c.edition.name


def x_edition__mutmut_5(obj: CoverBearer) -> str:
    """Return the edition name from a Cover-bearing type, defaulting to DEFAULT_EDITION."""
    c = cover_of(obj)
    if c is not None or not c.HasField("edition") or not c.edition.name:
        return DEFAULT_EDITION
    return c.edition.name


def x_edition__mutmut_6(obj: CoverBearer) -> str:
    """Return the edition name from a Cover-bearing type, defaulting to DEFAULT_EDITION."""
    c = cover_of(obj)
    if c is None or c.HasField("edition") or not c.edition.name:
        return DEFAULT_EDITION
    return c.edition.name


def x_edition__mutmut_7(obj: CoverBearer) -> str:
    """Return the edition name from a Cover-bearing type, defaulting to DEFAULT_EDITION."""
    c = cover_of(obj)
    if c is None or not c.HasField(None) or not c.edition.name:
        return DEFAULT_EDITION
    return c.edition.name


def x_edition__mutmut_8(obj: CoverBearer) -> str:
    """Return the edition name from a Cover-bearing type, defaulting to DEFAULT_EDITION."""
    c = cover_of(obj)
    if c is None or not c.HasField("XXeditionXX") or not c.edition.name:
        return DEFAULT_EDITION
    return c.edition.name


def x_edition__mutmut_9(obj: CoverBearer) -> str:
    """Return the edition name from a Cover-bearing type, defaulting to DEFAULT_EDITION."""
    c = cover_of(obj)
    if c is None or not c.HasField("EDITION") or not c.edition.name:
        return DEFAULT_EDITION
    return c.edition.name


def x_edition__mutmut_10(obj: CoverBearer) -> str:
    """Return the edition name from a Cover-bearing type, defaulting to DEFAULT_EDITION."""
    c = cover_of(obj)
    if c is None or not c.HasField("edition") or c.edition.name:
        return DEFAULT_EDITION
    return c.edition.name

x_edition__mutmut_mutants : ClassVar[MutantDict] = {
'x_edition__mutmut_1': x_edition__mutmut_1, 
    'x_edition__mutmut_2': x_edition__mutmut_2, 
    'x_edition__mutmut_3': x_edition__mutmut_3, 
    'x_edition__mutmut_4': x_edition__mutmut_4, 
    'x_edition__mutmut_5': x_edition__mutmut_5, 
    'x_edition__mutmut_6': x_edition__mutmut_6, 
    'x_edition__mutmut_7': x_edition__mutmut_7, 
    'x_edition__mutmut_8': x_edition__mutmut_8, 
    'x_edition__mutmut_9': x_edition__mutmut_9, 
    'x_edition__mutmut_10': x_edition__mutmut_10
}

def edition(*args, **kwargs):
    result = _mutmut_trampoline(x_edition__mutmut_orig, x_edition__mutmut_mutants, args, kwargs)
    return result 

edition.__signature__ = _mutmut_signature(x_edition__mutmut_orig)
x_edition__mutmut_orig.__name__ = 'x_edition'


def x_edition_opt__mutmut_orig(obj: CoverBearer) -> Optional[str]:
    """Return the edition name as Optional, None if not set."""
    c = cover_of(obj)
    if c is None or not c.HasField("edition") or not c.edition.name:
        return None
    return c.edition.name


def x_edition_opt__mutmut_1(obj: CoverBearer) -> Optional[str]:
    """Return the edition name as Optional, None if not set."""
    c = None
    if c is None or not c.HasField("edition") or not c.edition.name:
        return None
    return c.edition.name


def x_edition_opt__mutmut_2(obj: CoverBearer) -> Optional[str]:
    """Return the edition name as Optional, None if not set."""
    c = cover_of(None)
    if c is None or not c.HasField("edition") or not c.edition.name:
        return None
    return c.edition.name


def x_edition_opt__mutmut_3(obj: CoverBearer) -> Optional[str]:
    """Return the edition name as Optional, None if not set."""
    c = cover_of(obj)
    if c is None or not c.HasField("edition") and not c.edition.name:
        return None
    return c.edition.name


def x_edition_opt__mutmut_4(obj: CoverBearer) -> Optional[str]:
    """Return the edition name as Optional, None if not set."""
    c = cover_of(obj)
    if c is None and not c.HasField("edition") or not c.edition.name:
        return None
    return c.edition.name


def x_edition_opt__mutmut_5(obj: CoverBearer) -> Optional[str]:
    """Return the edition name as Optional, None if not set."""
    c = cover_of(obj)
    if c is not None or not c.HasField("edition") or not c.edition.name:
        return None
    return c.edition.name


def x_edition_opt__mutmut_6(obj: CoverBearer) -> Optional[str]:
    """Return the edition name as Optional, None if not set."""
    c = cover_of(obj)
    if c is None or c.HasField("edition") or not c.edition.name:
        return None
    return c.edition.name


def x_edition_opt__mutmut_7(obj: CoverBearer) -> Optional[str]:
    """Return the edition name as Optional, None if not set."""
    c = cover_of(obj)
    if c is None or not c.HasField(None) or not c.edition.name:
        return None
    return c.edition.name


def x_edition_opt__mutmut_8(obj: CoverBearer) -> Optional[str]:
    """Return the edition name as Optional, None if not set."""
    c = cover_of(obj)
    if c is None or not c.HasField("XXeditionXX") or not c.edition.name:
        return None
    return c.edition.name


def x_edition_opt__mutmut_9(obj: CoverBearer) -> Optional[str]:
    """Return the edition name as Optional, None if not set."""
    c = cover_of(obj)
    if c is None or not c.HasField("EDITION") or not c.edition.name:
        return None
    return c.edition.name


def x_edition_opt__mutmut_10(obj: CoverBearer) -> Optional[str]:
    """Return the edition name as Optional, None if not set."""
    c = cover_of(obj)
    if c is None or not c.HasField("edition") or c.edition.name:
        return None
    return c.edition.name

x_edition_opt__mutmut_mutants : ClassVar[MutantDict] = {
'x_edition_opt__mutmut_1': x_edition_opt__mutmut_1, 
    'x_edition_opt__mutmut_2': x_edition_opt__mutmut_2, 
    'x_edition_opt__mutmut_3': x_edition_opt__mutmut_3, 
    'x_edition_opt__mutmut_4': x_edition_opt__mutmut_4, 
    'x_edition_opt__mutmut_5': x_edition_opt__mutmut_5, 
    'x_edition_opt__mutmut_6': x_edition_opt__mutmut_6, 
    'x_edition_opt__mutmut_7': x_edition_opt__mutmut_7, 
    'x_edition_opt__mutmut_8': x_edition_opt__mutmut_8, 
    'x_edition_opt__mutmut_9': x_edition_opt__mutmut_9, 
    'x_edition_opt__mutmut_10': x_edition_opt__mutmut_10
}

def edition_opt(*args, **kwargs):
    result = _mutmut_trampoline(x_edition_opt__mutmut_orig, x_edition_opt__mutmut_mutants, args, kwargs)
    return result 

edition_opt.__signature__ = _mutmut_signature(x_edition_opt__mutmut_orig)
x_edition_opt__mutmut_orig.__name__ = 'x_edition_opt'


def x_routing_key__mutmut_orig(obj: CoverBearer) -> str:
    """Compute the bus routing key for a Cover-bearing type."""
    return domain(obj)


def x_routing_key__mutmut_1(obj: CoverBearer) -> str:
    """Compute the bus routing key for a Cover-bearing type."""
    return domain(None)

x_routing_key__mutmut_mutants : ClassVar[MutantDict] = {
'x_routing_key__mutmut_1': x_routing_key__mutmut_1
}

def routing_key(*args, **kwargs):
    result = _mutmut_trampoline(x_routing_key__mutmut_orig, x_routing_key__mutmut_mutants, args, kwargs)
    return result 

routing_key.__signature__ = _mutmut_signature(x_routing_key__mutmut_orig)
x_routing_key__mutmut_orig.__name__ = 'x_routing_key'


def x_cache_key__mutmut_orig(obj: CoverBearer) -> str:
    """Generate a cache key based on domain + root."""
    return f"{domain(obj)}:{root_id_hex(obj)}"


def x_cache_key__mutmut_1(obj: CoverBearer) -> str:
    """Generate a cache key based on domain + root."""
    return f"{domain(None)}:{root_id_hex(obj)}"


def x_cache_key__mutmut_2(obj: CoverBearer) -> str:
    """Generate a cache key based on domain + root."""
    return f"{domain(obj)}:{root_id_hex(None)}"

x_cache_key__mutmut_mutants : ClassVar[MutantDict] = {
'x_cache_key__mutmut_1': x_cache_key__mutmut_1, 
    'x_cache_key__mutmut_2': x_cache_key__mutmut_2
}

def cache_key(*args, **kwargs):
    result = _mutmut_trampoline(x_cache_key__mutmut_orig, x_cache_key__mutmut_mutants, args, kwargs)
    return result 

cache_key.__signature__ = _mutmut_signature(x_cache_key__mutmut_orig)
x_cache_key__mutmut_orig.__name__ = 'x_cache_key'


# UUID conversion


def x_uuid_to_proto__mutmut_orig(u: PyUUID) -> UUID:
    """Convert a Python UUID to a proto UUID."""
    return UUID(value=u.bytes)


# UUID conversion


def x_uuid_to_proto__mutmut_1(u: PyUUID) -> UUID:
    """Convert a Python UUID to a proto UUID."""
    return UUID(value=None)

x_uuid_to_proto__mutmut_mutants : ClassVar[MutantDict] = {
'x_uuid_to_proto__mutmut_1': x_uuid_to_proto__mutmut_1
}

def uuid_to_proto(*args, **kwargs):
    result = _mutmut_trampoline(x_uuid_to_proto__mutmut_orig, x_uuid_to_proto__mutmut_mutants, args, kwargs)
    return result 

uuid_to_proto.__signature__ = _mutmut_signature(x_uuid_to_proto__mutmut_orig)
x_uuid_to_proto__mutmut_orig.__name__ = 'x_uuid_to_proto'


def x_proto_to_uuid__mutmut_orig(u: UUID) -> PyUUID:
    """Convert a proto UUID to Python UUID."""
    return PyUUID(bytes=u.value)


def x_proto_to_uuid__mutmut_1(u: UUID) -> PyUUID:
    """Convert a proto UUID to Python UUID."""
    return PyUUID(bytes=None)

x_proto_to_uuid__mutmut_mutants : ClassVar[MutantDict] = {
'x_proto_to_uuid__mutmut_1': x_proto_to_uuid__mutmut_1
}

def proto_to_uuid(*args, **kwargs):
    result = _mutmut_trampoline(x_proto_to_uuid__mutmut_orig, x_proto_to_uuid__mutmut_mutants, args, kwargs)
    return result 

proto_to_uuid.__signature__ = _mutmut_signature(x_proto_to_uuid__mutmut_orig)
x_proto_to_uuid__mutmut_orig.__name__ = 'x_proto_to_uuid'


# Edition helpers


def x_main_timeline__mutmut_orig() -> Edition:
    """Return an Edition representing the main timeline."""
    return Edition(name=DEFAULT_EDITION)


# Edition helpers


def x_main_timeline__mutmut_1() -> Edition:
    """Return an Edition representing the main timeline."""
    return Edition(name=None)

x_main_timeline__mutmut_mutants : ClassVar[MutantDict] = {
'x_main_timeline__mutmut_1': x_main_timeline__mutmut_1
}

def main_timeline(*args, **kwargs):
    result = _mutmut_trampoline(x_main_timeline__mutmut_orig, x_main_timeline__mutmut_mutants, args, kwargs)
    return result 

main_timeline.__signature__ = _mutmut_signature(x_main_timeline__mutmut_orig)
x_main_timeline__mutmut_orig.__name__ = 'x_main_timeline'


def x_implicit_edition__mutmut_orig(name: str) -> Edition:
    """Create an edition with the given name but no divergences."""
    return Edition(name=name)


def x_implicit_edition__mutmut_1(name: str) -> Edition:
    """Create an edition with the given name but no divergences."""
    return Edition(name=None)

x_implicit_edition__mutmut_mutants : ClassVar[MutantDict] = {
'x_implicit_edition__mutmut_1': x_implicit_edition__mutmut_1
}

def implicit_edition(*args, **kwargs):
    result = _mutmut_trampoline(x_implicit_edition__mutmut_orig, x_implicit_edition__mutmut_mutants, args, kwargs)
    return result 

implicit_edition.__signature__ = _mutmut_signature(x_implicit_edition__mutmut_orig)
x_implicit_edition__mutmut_orig.__name__ = 'x_implicit_edition'


def x_explicit_edition__mutmut_orig(name: str, divergences: list[DomainDivergence]) -> Edition:
    """Create an edition with divergence points."""
    return Edition(name=name, divergences=divergences)


def x_explicit_edition__mutmut_1(name: str, divergences: list[DomainDivergence]) -> Edition:
    """Create an edition with divergence points."""
    return Edition(name=None, divergences=divergences)


def x_explicit_edition__mutmut_2(name: str, divergences: list[DomainDivergence]) -> Edition:
    """Create an edition with divergence points."""
    return Edition(name=name, divergences=None)


def x_explicit_edition__mutmut_3(name: str, divergences: list[DomainDivergence]) -> Edition:
    """Create an edition with divergence points."""
    return Edition(divergences=divergences)


def x_explicit_edition__mutmut_4(name: str, divergences: list[DomainDivergence]) -> Edition:
    """Create an edition with divergence points."""
    return Edition(name=name, )

x_explicit_edition__mutmut_mutants : ClassVar[MutantDict] = {
'x_explicit_edition__mutmut_1': x_explicit_edition__mutmut_1, 
    'x_explicit_edition__mutmut_2': x_explicit_edition__mutmut_2, 
    'x_explicit_edition__mutmut_3': x_explicit_edition__mutmut_3, 
    'x_explicit_edition__mutmut_4': x_explicit_edition__mutmut_4
}

def explicit_edition(*args, **kwargs):
    result = _mutmut_trampoline(x_explicit_edition__mutmut_orig, x_explicit_edition__mutmut_mutants, args, kwargs)
    return result 

explicit_edition.__signature__ = _mutmut_signature(x_explicit_edition__mutmut_orig)
x_explicit_edition__mutmut_orig.__name__ = 'x_explicit_edition'


def x_is_main_timeline__mutmut_orig(e: Optional[Edition]) -> bool:
    """Check if an edition represents the main timeline."""
    return e is None or not e.name or e.name == DEFAULT_EDITION


def x_is_main_timeline__mutmut_1(e: Optional[Edition]) -> bool:
    """Check if an edition represents the main timeline."""
    return e is None or not e.name and e.name == DEFAULT_EDITION


def x_is_main_timeline__mutmut_2(e: Optional[Edition]) -> bool:
    """Check if an edition represents the main timeline."""
    return e is None and not e.name or e.name == DEFAULT_EDITION


def x_is_main_timeline__mutmut_3(e: Optional[Edition]) -> bool:
    """Check if an edition represents the main timeline."""
    return e is not None or not e.name or e.name == DEFAULT_EDITION


def x_is_main_timeline__mutmut_4(e: Optional[Edition]) -> bool:
    """Check if an edition represents the main timeline."""
    return e is None or e.name or e.name == DEFAULT_EDITION


def x_is_main_timeline__mutmut_5(e: Optional[Edition]) -> bool:
    """Check if an edition represents the main timeline."""
    return e is None or not e.name or e.name != DEFAULT_EDITION

x_is_main_timeline__mutmut_mutants : ClassVar[MutantDict] = {
'x_is_main_timeline__mutmut_1': x_is_main_timeline__mutmut_1, 
    'x_is_main_timeline__mutmut_2': x_is_main_timeline__mutmut_2, 
    'x_is_main_timeline__mutmut_3': x_is_main_timeline__mutmut_3, 
    'x_is_main_timeline__mutmut_4': x_is_main_timeline__mutmut_4, 
    'x_is_main_timeline__mutmut_5': x_is_main_timeline__mutmut_5
}

def is_main_timeline(*args, **kwargs):
    result = _mutmut_trampoline(x_is_main_timeline__mutmut_orig, x_is_main_timeline__mutmut_mutants, args, kwargs)
    return result 

is_main_timeline.__signature__ = _mutmut_signature(x_is_main_timeline__mutmut_orig)
x_is_main_timeline__mutmut_orig.__name__ = 'x_is_main_timeline'


def x_divergence_for__mutmut_orig(e: Optional[Edition], domain_name: str) -> int:
    """Return the divergence sequence for a domain, or -1 if not found."""
    if e is None:
        return -1
    for d in e.divergences:
        if d.domain == domain_name:
            return d.sequence
    return -1


def x_divergence_for__mutmut_1(e: Optional[Edition], domain_name: str) -> int:
    """Return the divergence sequence for a domain, or -1 if not found."""
    if e is not None:
        return -1
    for d in e.divergences:
        if d.domain == domain_name:
            return d.sequence
    return -1


def x_divergence_for__mutmut_2(e: Optional[Edition], domain_name: str) -> int:
    """Return the divergence sequence for a domain, or -1 if not found."""
    if e is None:
        return +1
    for d in e.divergences:
        if d.domain == domain_name:
            return d.sequence
    return -1


def x_divergence_for__mutmut_3(e: Optional[Edition], domain_name: str) -> int:
    """Return the divergence sequence for a domain, or -1 if not found."""
    if e is None:
        return -2
    for d in e.divergences:
        if d.domain == domain_name:
            return d.sequence
    return -1


def x_divergence_for__mutmut_4(e: Optional[Edition], domain_name: str) -> int:
    """Return the divergence sequence for a domain, or -1 if not found."""
    if e is None:
        return -1
    for d in e.divergences:
        if d.domain != domain_name:
            return d.sequence
    return -1


def x_divergence_for__mutmut_5(e: Optional[Edition], domain_name: str) -> int:
    """Return the divergence sequence for a domain, or -1 if not found."""
    if e is None:
        return -1
    for d in e.divergences:
        if d.domain == domain_name:
            return d.sequence
    return +1


def x_divergence_for__mutmut_6(e: Optional[Edition], domain_name: str) -> int:
    """Return the divergence sequence for a domain, or -1 if not found."""
    if e is None:
        return -1
    for d in e.divergences:
        if d.domain == domain_name:
            return d.sequence
    return -2

x_divergence_for__mutmut_mutants : ClassVar[MutantDict] = {
'x_divergence_for__mutmut_1': x_divergence_for__mutmut_1, 
    'x_divergence_for__mutmut_2': x_divergence_for__mutmut_2, 
    'x_divergence_for__mutmut_3': x_divergence_for__mutmut_3, 
    'x_divergence_for__mutmut_4': x_divergence_for__mutmut_4, 
    'x_divergence_for__mutmut_5': x_divergence_for__mutmut_5, 
    'x_divergence_for__mutmut_6': x_divergence_for__mutmut_6
}

def divergence_for(*args, **kwargs):
    result = _mutmut_trampoline(x_divergence_for__mutmut_orig, x_divergence_for__mutmut_mutants, args, kwargs)
    return result 

divergence_for.__signature__ = _mutmut_signature(x_divergence_for__mutmut_orig)
x_divergence_for__mutmut_orig.__name__ = 'x_divergence_for'


# EventBook helpers


def x_next_sequence__mutmut_orig(book: EventBook) -> int:
    """Return the next sequence number from an EventBook.

    The framework computes this value on load.
    """
    if book is None:
        return 0
    return book.next_sequence


# EventBook helpers


def x_next_sequence__mutmut_1(book: EventBook) -> int:
    """Return the next sequence number from an EventBook.

    The framework computes this value on load.
    """
    if book is not None:
        return 0
    return book.next_sequence


# EventBook helpers


def x_next_sequence__mutmut_2(book: EventBook) -> int:
    """Return the next sequence number from an EventBook.

    The framework computes this value on load.
    """
    if book is None:
        return 1
    return book.next_sequence

x_next_sequence__mutmut_mutants : ClassVar[MutantDict] = {
'x_next_sequence__mutmut_1': x_next_sequence__mutmut_1, 
    'x_next_sequence__mutmut_2': x_next_sequence__mutmut_2
}

def next_sequence(*args, **kwargs):
    result = _mutmut_trampoline(x_next_sequence__mutmut_orig, x_next_sequence__mutmut_mutants, args, kwargs)
    return result 

next_sequence.__signature__ = _mutmut_signature(x_next_sequence__mutmut_orig)
x_next_sequence__mutmut_orig.__name__ = 'x_next_sequence'


def x_event_pages__mutmut_orig(book: Optional[EventBook]) -> list[EventPage]:
    """Return the event pages from an EventBook, or empty list if None."""
    if book is None:
        return []
    return list(book.pages)


def x_event_pages__mutmut_1(book: Optional[EventBook]) -> list[EventPage]:
    """Return the event pages from an EventBook, or empty list if None."""
    if book is not None:
        return []
    return list(book.pages)


def x_event_pages__mutmut_2(book: Optional[EventBook]) -> list[EventPage]:
    """Return the event pages from an EventBook, or empty list if None."""
    if book is None:
        return []
    return list(None)

x_event_pages__mutmut_mutants : ClassVar[MutantDict] = {
'x_event_pages__mutmut_1': x_event_pages__mutmut_1, 
    'x_event_pages__mutmut_2': x_event_pages__mutmut_2
}

def event_pages(*args, **kwargs):
    result = _mutmut_trampoline(x_event_pages__mutmut_orig, x_event_pages__mutmut_mutants, args, kwargs)
    return result 

event_pages.__signature__ = _mutmut_signature(x_event_pages__mutmut_orig)
x_event_pages__mutmut_orig.__name__ = 'x_event_pages'


# CommandBook helpers


def x_command_pages__mutmut_orig(book: Optional[CommandBook]) -> list[CommandPage]:
    """Return the command pages from a CommandBook, or empty list if None."""
    if book is None:
        return []
    return list(book.pages)


# CommandBook helpers


def x_command_pages__mutmut_1(book: Optional[CommandBook]) -> list[CommandPage]:
    """Return the command pages from a CommandBook, or empty list if None."""
    if book is not None:
        return []
    return list(book.pages)


# CommandBook helpers


def x_command_pages__mutmut_2(book: Optional[CommandBook]) -> list[CommandPage]:
    """Return the command pages from a CommandBook, or empty list if None."""
    if book is None:
        return []
    return list(None)

x_command_pages__mutmut_mutants : ClassVar[MutantDict] = {
'x_command_pages__mutmut_1': x_command_pages__mutmut_1, 
    'x_command_pages__mutmut_2': x_command_pages__mutmut_2
}

def command_pages(*args, **kwargs):
    result = _mutmut_trampoline(x_command_pages__mutmut_orig, x_command_pages__mutmut_mutants, args, kwargs)
    return result 

command_pages.__signature__ = _mutmut_signature(x_command_pages__mutmut_orig)
x_command_pages__mutmut_orig.__name__ = 'x_command_pages'


# CommandResponse helpers


def x_events_from_response__mutmut_orig(resp) -> list[EventPage]:
    """Extract the event pages from a CommandResponse."""
    if resp is None or not resp.HasField("events"):
        return []
    return list(resp.events.pages)


# CommandResponse helpers


def x_events_from_response__mutmut_1(resp) -> list[EventPage]:
    """Extract the event pages from a CommandResponse."""
    if resp is None and not resp.HasField("events"):
        return []
    return list(resp.events.pages)


# CommandResponse helpers


def x_events_from_response__mutmut_2(resp) -> list[EventPage]:
    """Extract the event pages from a CommandResponse."""
    if resp is not None or not resp.HasField("events"):
        return []
    return list(resp.events.pages)


# CommandResponse helpers


def x_events_from_response__mutmut_3(resp) -> list[EventPage]:
    """Extract the event pages from a CommandResponse."""
    if resp is None or resp.HasField("events"):
        return []
    return list(resp.events.pages)


# CommandResponse helpers


def x_events_from_response__mutmut_4(resp) -> list[EventPage]:
    """Extract the event pages from a CommandResponse."""
    if resp is None or not resp.HasField(None):
        return []
    return list(resp.events.pages)


# CommandResponse helpers


def x_events_from_response__mutmut_5(resp) -> list[EventPage]:
    """Extract the event pages from a CommandResponse."""
    if resp is None or not resp.HasField("XXeventsXX"):
        return []
    return list(resp.events.pages)


# CommandResponse helpers


def x_events_from_response__mutmut_6(resp) -> list[EventPage]:
    """Extract the event pages from a CommandResponse."""
    if resp is None or not resp.HasField("EVENTS"):
        return []
    return list(resp.events.pages)


# CommandResponse helpers


def x_events_from_response__mutmut_7(resp) -> list[EventPage]:
    """Extract the event pages from a CommandResponse."""
    if resp is None or not resp.HasField("events"):
        return []
    return list(None)

x_events_from_response__mutmut_mutants : ClassVar[MutantDict] = {
'x_events_from_response__mutmut_1': x_events_from_response__mutmut_1, 
    'x_events_from_response__mutmut_2': x_events_from_response__mutmut_2, 
    'x_events_from_response__mutmut_3': x_events_from_response__mutmut_3, 
    'x_events_from_response__mutmut_4': x_events_from_response__mutmut_4, 
    'x_events_from_response__mutmut_5': x_events_from_response__mutmut_5, 
    'x_events_from_response__mutmut_6': x_events_from_response__mutmut_6, 
    'x_events_from_response__mutmut_7': x_events_from_response__mutmut_7
}

def events_from_response(*args, **kwargs):
    result = _mutmut_trampoline(x_events_from_response__mutmut_orig, x_events_from_response__mutmut_mutants, args, kwargs)
    return result 

events_from_response.__signature__ = _mutmut_signature(x_events_from_response__mutmut_orig)
x_events_from_response__mutmut_orig.__name__ = 'x_events_from_response'


# Type URL helpers


def type_url(package_name: str, type_name: str) -> str:
    """Construct a full type URL from a package and type name."""
    return f"{TYPE_URL_PREFIX}{package_name}.{type_name}"


def x_type_name_from_url__mutmut_orig(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_1(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "XX.XX" in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_2(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." not in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_3(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(None, 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_4(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", None)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_5(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_6(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", )[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_7(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.split(".", 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_8(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit("XX.XX", 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_9(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 2)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_10(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 1)[2]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_11(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "XX/XX" in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_12(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "/" not in type_url_str:
        return type_url_str.rsplit("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_13(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit(None, 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_14(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", None)[1]
    return type_url_str


def x_type_name_from_url__mutmut_15(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit(1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_16(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", )[1]
    return type_url_str


def x_type_name_from_url__mutmut_17(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "/" in type_url_str:
        return type_url_str.split("/", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_18(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("XX/XX", 1)[1]
    return type_url_str


def x_type_name_from_url__mutmut_19(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 2)[1]
    return type_url_str


def x_type_name_from_url__mutmut_20(type_url_str: str) -> str:
    """Extract the type name from a type URL."""
    if "." in type_url_str:
        return type_url_str.rsplit(".", 1)[1]
    if "/" in type_url_str:
        return type_url_str.rsplit("/", 1)[2]
    return type_url_str

x_type_name_from_url__mutmut_mutants : ClassVar[MutantDict] = {
'x_type_name_from_url__mutmut_1': x_type_name_from_url__mutmut_1, 
    'x_type_name_from_url__mutmut_2': x_type_name_from_url__mutmut_2, 
    'x_type_name_from_url__mutmut_3': x_type_name_from_url__mutmut_3, 
    'x_type_name_from_url__mutmut_4': x_type_name_from_url__mutmut_4, 
    'x_type_name_from_url__mutmut_5': x_type_name_from_url__mutmut_5, 
    'x_type_name_from_url__mutmut_6': x_type_name_from_url__mutmut_6, 
    'x_type_name_from_url__mutmut_7': x_type_name_from_url__mutmut_7, 
    'x_type_name_from_url__mutmut_8': x_type_name_from_url__mutmut_8, 
    'x_type_name_from_url__mutmut_9': x_type_name_from_url__mutmut_9, 
    'x_type_name_from_url__mutmut_10': x_type_name_from_url__mutmut_10, 
    'x_type_name_from_url__mutmut_11': x_type_name_from_url__mutmut_11, 
    'x_type_name_from_url__mutmut_12': x_type_name_from_url__mutmut_12, 
    'x_type_name_from_url__mutmut_13': x_type_name_from_url__mutmut_13, 
    'x_type_name_from_url__mutmut_14': x_type_name_from_url__mutmut_14, 
    'x_type_name_from_url__mutmut_15': x_type_name_from_url__mutmut_15, 
    'x_type_name_from_url__mutmut_16': x_type_name_from_url__mutmut_16, 
    'x_type_name_from_url__mutmut_17': x_type_name_from_url__mutmut_17, 
    'x_type_name_from_url__mutmut_18': x_type_name_from_url__mutmut_18, 
    'x_type_name_from_url__mutmut_19': x_type_name_from_url__mutmut_19, 
    'x_type_name_from_url__mutmut_20': x_type_name_from_url__mutmut_20
}

def type_name_from_url(*args, **kwargs):
    result = _mutmut_trampoline(x_type_name_from_url__mutmut_orig, x_type_name_from_url__mutmut_mutants, args, kwargs)
    return result 

type_name_from_url.__signature__ = _mutmut_signature(x_type_name_from_url__mutmut_orig)
x_type_name_from_url__mutmut_orig.__name__ = 'x_type_name_from_url'


def x_type_url_matches__mutmut_orig(type_url_str: str, suffix: str) -> bool:
    """Check if a type URL ends with the given suffix."""
    return type_url_str.endswith(suffix)


def x_type_url_matches__mutmut_1(type_url_str: str, suffix: str) -> bool:
    """Check if a type URL ends with the given suffix."""
    return type_url_str.endswith(None)

x_type_url_matches__mutmut_mutants : ClassVar[MutantDict] = {
'x_type_url_matches__mutmut_1': x_type_url_matches__mutmut_1
}

def type_url_matches(*args, **kwargs):
    result = _mutmut_trampoline(x_type_url_matches__mutmut_orig, x_type_url_matches__mutmut_mutants, args, kwargs)
    return result 

type_url_matches.__signature__ = _mutmut_signature(x_type_url_matches__mutmut_orig)
x_type_url_matches__mutmut_orig.__name__ = 'x_type_url_matches'


# Timestamp helpers


def x_now__mutmut_orig() -> Timestamp:
    """Return the current time as a protobuf Timestamp."""
    ts = Timestamp()
    ts.GetCurrentTime()
    return ts


# Timestamp helpers


def x_now__mutmut_1() -> Timestamp:
    """Return the current time as a protobuf Timestamp."""
    ts = None
    ts.GetCurrentTime()
    return ts

x_now__mutmut_mutants : ClassVar[MutantDict] = {
'x_now__mutmut_1': x_now__mutmut_1
}

def now(*args, **kwargs):
    result = _mutmut_trampoline(x_now__mutmut_orig, x_now__mutmut_mutants, args, kwargs)
    return result 

now.__signature__ = _mutmut_signature(x_now__mutmut_orig)
x_now__mutmut_orig.__name__ = 'x_now'


def x_parse_timestamp__mutmut_orig(rfc3339: str) -> Timestamp:
    """Parse an RFC3339 timestamp string."""
    try:
        ts = Timestamp()
        ts.FromJsonString(rfc3339)
        return ts
    except ValueError as e:
        raise InvalidTimestampError(str(e)) from e


def x_parse_timestamp__mutmut_1(rfc3339: str) -> Timestamp:
    """Parse an RFC3339 timestamp string."""
    try:
        ts = None
        ts.FromJsonString(rfc3339)
        return ts
    except ValueError as e:
        raise InvalidTimestampError(str(e)) from e


def x_parse_timestamp__mutmut_2(rfc3339: str) -> Timestamp:
    """Parse an RFC3339 timestamp string."""
    try:
        ts = Timestamp()
        ts.FromJsonString(None)
        return ts
    except ValueError as e:
        raise InvalidTimestampError(str(e)) from e


def x_parse_timestamp__mutmut_3(rfc3339: str) -> Timestamp:
    """Parse an RFC3339 timestamp string."""
    try:
        ts = Timestamp()
        ts.FromJsonString(rfc3339)
        return ts
    except ValueError as e:
        raise InvalidTimestampError(None) from e


def x_parse_timestamp__mutmut_4(rfc3339: str) -> Timestamp:
    """Parse an RFC3339 timestamp string."""
    try:
        ts = Timestamp()
        ts.FromJsonString(rfc3339)
        return ts
    except ValueError as e:
        raise InvalidTimestampError(str(None)) from e

x_parse_timestamp__mutmut_mutants : ClassVar[MutantDict] = {
'x_parse_timestamp__mutmut_1': x_parse_timestamp__mutmut_1, 
    'x_parse_timestamp__mutmut_2': x_parse_timestamp__mutmut_2, 
    'x_parse_timestamp__mutmut_3': x_parse_timestamp__mutmut_3, 
    'x_parse_timestamp__mutmut_4': x_parse_timestamp__mutmut_4
}

def parse_timestamp(*args, **kwargs):
    result = _mutmut_trampoline(x_parse_timestamp__mutmut_orig, x_parse_timestamp__mutmut_mutants, args, kwargs)
    return result 

parse_timestamp.__signature__ = _mutmut_signature(x_parse_timestamp__mutmut_orig)
x_parse_timestamp__mutmut_orig.__name__ = 'x_parse_timestamp'


# Event decoding


def x_decode_event__mutmut_orig(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None or not page.HasField("event"):
        return None
    if not type_url_matches(page.event.type_url, type_suffix):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_1(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None and not page.HasField("event"):
        return None
    if not type_url_matches(page.event.type_url, type_suffix):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_2(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is not None or not page.HasField("event"):
        return None
    if not type_url_matches(page.event.type_url, type_suffix):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_3(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None or page.HasField("event"):
        return None
    if not type_url_matches(page.event.type_url, type_suffix):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_4(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None or not page.HasField(None):
        return None
    if not type_url_matches(page.event.type_url, type_suffix):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_5(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None or not page.HasField("XXeventXX"):
        return None
    if not type_url_matches(page.event.type_url, type_suffix):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_6(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None or not page.HasField("EVENT"):
        return None
    if not type_url_matches(page.event.type_url, type_suffix):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_7(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None or not page.HasField("event"):
        return None
    if type_url_matches(page.event.type_url, type_suffix):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_8(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None or not page.HasField("event"):
        return None
    if not type_url_matches(None, type_suffix):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_9(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None or not page.HasField("event"):
        return None
    if not type_url_matches(page.event.type_url, None):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_10(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None or not page.HasField("event"):
        return None
    if not type_url_matches(type_suffix):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_11(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None or not page.HasField("event"):
        return None
    if not type_url_matches(page.event.type_url, ):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_12(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None or not page.HasField("event"):
        return None
    if not type_url_matches(page.event.type_url, type_suffix):
        return None
    try:
        msg = None
        page.event.Unpack(msg)
        return msg
    except Exception:
        return None


# Event decoding


def x_decode_event__mutmut_13(page: EventPage, type_suffix: str, msg_class) -> Optional[object]:
    """Attempt to decode an event payload if the type URL matches.

    Args:
        page: The event page to decode
        type_suffix: The expected type URL suffix
        msg_class: The protobuf message class to decode into

    Returns:
        The decoded message if type matches and decoding succeeds, None otherwise
    """
    if page is None or not page.HasField("event"):
        return None
    if not type_url_matches(page.event.type_url, type_suffix):
        return None
    try:
        msg = msg_class()
        page.event.Unpack(None)
        return msg
    except Exception:
        return None

x_decode_event__mutmut_mutants : ClassVar[MutantDict] = {
'x_decode_event__mutmut_1': x_decode_event__mutmut_1, 
    'x_decode_event__mutmut_2': x_decode_event__mutmut_2, 
    'x_decode_event__mutmut_3': x_decode_event__mutmut_3, 
    'x_decode_event__mutmut_4': x_decode_event__mutmut_4, 
    'x_decode_event__mutmut_5': x_decode_event__mutmut_5, 
    'x_decode_event__mutmut_6': x_decode_event__mutmut_6, 
    'x_decode_event__mutmut_7': x_decode_event__mutmut_7, 
    'x_decode_event__mutmut_8': x_decode_event__mutmut_8, 
    'x_decode_event__mutmut_9': x_decode_event__mutmut_9, 
    'x_decode_event__mutmut_10': x_decode_event__mutmut_10, 
    'x_decode_event__mutmut_11': x_decode_event__mutmut_11, 
    'x_decode_event__mutmut_12': x_decode_event__mutmut_12, 
    'x_decode_event__mutmut_13': x_decode_event__mutmut_13
}

def decode_event(*args, **kwargs):
    result = _mutmut_trampoline(x_decode_event__mutmut_orig, x_decode_event__mutmut_mutants, args, kwargs)
    return result 

decode_event.__signature__ = _mutmut_signature(x_decode_event__mutmut_orig)
x_decode_event__mutmut_orig.__name__ = 'x_decode_event'


# Construction helpers


def x_new_cover__mutmut_orig(
    domain_name: str,
    root: PyUUID,
    correlation_id_val: str = "",
    edition_val: Optional[Edition] = None,
) -> Cover:
    """Create a new Cover with the given parameters."""
    cover = Cover(
        domain=domain_name,
        root=uuid_to_proto(root),
        correlation_id=correlation_id_val,
    )
    if edition_val is not None:
        cover.edition.CopyFrom(edition_val)
    return cover


# Construction helpers


def x_new_cover__mutmut_1(
    domain_name: str,
    root: PyUUID,
    correlation_id_val: str = "XXXX",
    edition_val: Optional[Edition] = None,
) -> Cover:
    """Create a new Cover with the given parameters."""
    cover = Cover(
        domain=domain_name,
        root=uuid_to_proto(root),
        correlation_id=correlation_id_val,
    )
    if edition_val is not None:
        cover.edition.CopyFrom(edition_val)
    return cover


# Construction helpers


def x_new_cover__mutmut_2(
    domain_name: str,
    root: PyUUID,
    correlation_id_val: str = "",
    edition_val: Optional[Edition] = None,
) -> Cover:
    """Create a new Cover with the given parameters."""
    cover = None
    if edition_val is not None:
        cover.edition.CopyFrom(edition_val)
    return cover


# Construction helpers


def x_new_cover__mutmut_3(
    domain_name: str,
    root: PyUUID,
    correlation_id_val: str = "",
    edition_val: Optional[Edition] = None,
) -> Cover:
    """Create a new Cover with the given parameters."""
    cover = Cover(
        domain=None,
        root=uuid_to_proto(root),
        correlation_id=correlation_id_val,
    )
    if edition_val is not None:
        cover.edition.CopyFrom(edition_val)
    return cover


# Construction helpers


def x_new_cover__mutmut_4(
    domain_name: str,
    root: PyUUID,
    correlation_id_val: str = "",
    edition_val: Optional[Edition] = None,
) -> Cover:
    """Create a new Cover with the given parameters."""
    cover = Cover(
        domain=domain_name,
        root=None,
        correlation_id=correlation_id_val,
    )
    if edition_val is not None:
        cover.edition.CopyFrom(edition_val)
    return cover


# Construction helpers


def x_new_cover__mutmut_5(
    domain_name: str,
    root: PyUUID,
    correlation_id_val: str = "",
    edition_val: Optional[Edition] = None,
) -> Cover:
    """Create a new Cover with the given parameters."""
    cover = Cover(
        domain=domain_name,
        root=uuid_to_proto(root),
        correlation_id=None,
    )
    if edition_val is not None:
        cover.edition.CopyFrom(edition_val)
    return cover


# Construction helpers


def x_new_cover__mutmut_6(
    domain_name: str,
    root: PyUUID,
    correlation_id_val: str = "",
    edition_val: Optional[Edition] = None,
) -> Cover:
    """Create a new Cover with the given parameters."""
    cover = Cover(
        root=uuid_to_proto(root),
        correlation_id=correlation_id_val,
    )
    if edition_val is not None:
        cover.edition.CopyFrom(edition_val)
    return cover


# Construction helpers


def x_new_cover__mutmut_7(
    domain_name: str,
    root: PyUUID,
    correlation_id_val: str = "",
    edition_val: Optional[Edition] = None,
) -> Cover:
    """Create a new Cover with the given parameters."""
    cover = Cover(
        domain=domain_name,
        correlation_id=correlation_id_val,
    )
    if edition_val is not None:
        cover.edition.CopyFrom(edition_val)
    return cover


# Construction helpers


def x_new_cover__mutmut_8(
    domain_name: str,
    root: PyUUID,
    correlation_id_val: str = "",
    edition_val: Optional[Edition] = None,
) -> Cover:
    """Create a new Cover with the given parameters."""
    cover = Cover(
        domain=domain_name,
        root=uuid_to_proto(root),
        )
    if edition_val is not None:
        cover.edition.CopyFrom(edition_val)
    return cover


# Construction helpers


def x_new_cover__mutmut_9(
    domain_name: str,
    root: PyUUID,
    correlation_id_val: str = "",
    edition_val: Optional[Edition] = None,
) -> Cover:
    """Create a new Cover with the given parameters."""
    cover = Cover(
        domain=domain_name,
        root=uuid_to_proto(None),
        correlation_id=correlation_id_val,
    )
    if edition_val is not None:
        cover.edition.CopyFrom(edition_val)
    return cover


# Construction helpers


def x_new_cover__mutmut_10(
    domain_name: str,
    root: PyUUID,
    correlation_id_val: str = "",
    edition_val: Optional[Edition] = None,
) -> Cover:
    """Create a new Cover with the given parameters."""
    cover = Cover(
        domain=domain_name,
        root=uuid_to_proto(root),
        correlation_id=correlation_id_val,
    )
    if edition_val is None:
        cover.edition.CopyFrom(edition_val)
    return cover


# Construction helpers


def x_new_cover__mutmut_11(
    domain_name: str,
    root: PyUUID,
    correlation_id_val: str = "",
    edition_val: Optional[Edition] = None,
) -> Cover:
    """Create a new Cover with the given parameters."""
    cover = Cover(
        domain=domain_name,
        root=uuid_to_proto(root),
        correlation_id=correlation_id_val,
    )
    if edition_val is not None:
        cover.edition.CopyFrom(None)
    return cover

x_new_cover__mutmut_mutants : ClassVar[MutantDict] = {
'x_new_cover__mutmut_1': x_new_cover__mutmut_1, 
    'x_new_cover__mutmut_2': x_new_cover__mutmut_2, 
    'x_new_cover__mutmut_3': x_new_cover__mutmut_3, 
    'x_new_cover__mutmut_4': x_new_cover__mutmut_4, 
    'x_new_cover__mutmut_5': x_new_cover__mutmut_5, 
    'x_new_cover__mutmut_6': x_new_cover__mutmut_6, 
    'x_new_cover__mutmut_7': x_new_cover__mutmut_7, 
    'x_new_cover__mutmut_8': x_new_cover__mutmut_8, 
    'x_new_cover__mutmut_9': x_new_cover__mutmut_9, 
    'x_new_cover__mutmut_10': x_new_cover__mutmut_10, 
    'x_new_cover__mutmut_11': x_new_cover__mutmut_11
}

def new_cover(*args, **kwargs):
    result = _mutmut_trampoline(x_new_cover__mutmut_orig, x_new_cover__mutmut_mutants, args, kwargs)
    return result 

new_cover.__signature__ = _mutmut_signature(x_new_cover__mutmut_orig)
x_new_cover__mutmut_orig.__name__ = 'x_new_cover'


def x_new_command_page__mutmut_orig(sequence: int, command: ProtoAny) -> CommandPage:
    """Create a command page from a sequence and Any message."""
    page = CommandPage(sequence=sequence)
    page.command.CopyFrom(command)
    return page


def x_new_command_page__mutmut_1(sequence: int, command: ProtoAny) -> CommandPage:
    """Create a command page from a sequence and Any message."""
    page = None
    page.command.CopyFrom(command)
    return page


def x_new_command_page__mutmut_2(sequence: int, command: ProtoAny) -> CommandPage:
    """Create a command page from a sequence and Any message."""
    page = CommandPage(sequence=None)
    page.command.CopyFrom(command)
    return page


def x_new_command_page__mutmut_3(sequence: int, command: ProtoAny) -> CommandPage:
    """Create a command page from a sequence and Any message."""
    page = CommandPage(sequence=sequence)
    page.command.CopyFrom(None)
    return page

x_new_command_page__mutmut_mutants : ClassVar[MutantDict] = {
'x_new_command_page__mutmut_1': x_new_command_page__mutmut_1, 
    'x_new_command_page__mutmut_2': x_new_command_page__mutmut_2, 
    'x_new_command_page__mutmut_3': x_new_command_page__mutmut_3
}

def new_command_page(*args, **kwargs):
    result = _mutmut_trampoline(x_new_command_page__mutmut_orig, x_new_command_page__mutmut_mutants, args, kwargs)
    return result 

new_command_page.__signature__ = _mutmut_signature(x_new_command_page__mutmut_orig)
x_new_command_page__mutmut_orig.__name__ = 'x_new_command_page'


def x_new_command_book__mutmut_orig(cover: Cover, pages: list[CommandPage]) -> CommandBook:
    """Create a CommandBook with the given cover and pages."""
    book = CommandBook()
    book.cover.CopyFrom(cover)
    book.pages.extend(pages)
    return book


def x_new_command_book__mutmut_1(cover: Cover, pages: list[CommandPage]) -> CommandBook:
    """Create a CommandBook with the given cover and pages."""
    book = None
    book.cover.CopyFrom(cover)
    book.pages.extend(pages)
    return book


def x_new_command_book__mutmut_2(cover: Cover, pages: list[CommandPage]) -> CommandBook:
    """Create a CommandBook with the given cover and pages."""
    book = CommandBook()
    book.cover.CopyFrom(None)
    book.pages.extend(pages)
    return book


def x_new_command_book__mutmut_3(cover: Cover, pages: list[CommandPage]) -> CommandBook:
    """Create a CommandBook with the given cover and pages."""
    book = CommandBook()
    book.cover.CopyFrom(cover)
    book.pages.extend(None)
    return book

x_new_command_book__mutmut_mutants : ClassVar[MutantDict] = {
'x_new_command_book__mutmut_1': x_new_command_book__mutmut_1, 
    'x_new_command_book__mutmut_2': x_new_command_book__mutmut_2, 
    'x_new_command_book__mutmut_3': x_new_command_book__mutmut_3
}

def new_command_book(*args, **kwargs):
    result = _mutmut_trampoline(x_new_command_book__mutmut_orig, x_new_command_book__mutmut_mutants, args, kwargs)
    return result 

new_command_book.__signature__ = _mutmut_signature(x_new_command_book__mutmut_orig)
x_new_command_book__mutmut_orig.__name__ = 'x_new_command_book'


def x_range_selection__mutmut_orig(lower: int, upper: Optional[int] = None) -> SequenceRange:
    """Create a sequence range selection."""
    r = SequenceRange(lower=lower)
    if upper is not None:
        r.upper = upper
    return r


def x_range_selection__mutmut_1(lower: int, upper: Optional[int] = None) -> SequenceRange:
    """Create a sequence range selection."""
    r = None
    if upper is not None:
        r.upper = upper
    return r


def x_range_selection__mutmut_2(lower: int, upper: Optional[int] = None) -> SequenceRange:
    """Create a sequence range selection."""
    r = SequenceRange(lower=None)
    if upper is not None:
        r.upper = upper
    return r


def x_range_selection__mutmut_3(lower: int, upper: Optional[int] = None) -> SequenceRange:
    """Create a sequence range selection."""
    r = SequenceRange(lower=lower)
    if upper is None:
        r.upper = upper
    return r


def x_range_selection__mutmut_4(lower: int, upper: Optional[int] = None) -> SequenceRange:
    """Create a sequence range selection."""
    r = SequenceRange(lower=lower)
    if upper is not None:
        r.upper = None
    return r

x_range_selection__mutmut_mutants : ClassVar[MutantDict] = {
'x_range_selection__mutmut_1': x_range_selection__mutmut_1, 
    'x_range_selection__mutmut_2': x_range_selection__mutmut_2, 
    'x_range_selection__mutmut_3': x_range_selection__mutmut_3, 
    'x_range_selection__mutmut_4': x_range_selection__mutmut_4
}

def range_selection(*args, **kwargs):
    result = _mutmut_trampoline(x_range_selection__mutmut_orig, x_range_selection__mutmut_mutants, args, kwargs)
    return result 

range_selection.__signature__ = _mutmut_signature(x_range_selection__mutmut_orig)
x_range_selection__mutmut_orig.__name__ = 'x_range_selection'


def x_temporal_by_sequence__mutmut_orig(seq: int) -> TemporalQuery:
    """Create a temporal selection as-of a sequence."""
    return TemporalQuery(as_of_sequence=seq)


def x_temporal_by_sequence__mutmut_1(seq: int) -> TemporalQuery:
    """Create a temporal selection as-of a sequence."""
    return TemporalQuery(as_of_sequence=None)

x_temporal_by_sequence__mutmut_mutants : ClassVar[MutantDict] = {
'x_temporal_by_sequence__mutmut_1': x_temporal_by_sequence__mutmut_1
}

def temporal_by_sequence(*args, **kwargs):
    result = _mutmut_trampoline(x_temporal_by_sequence__mutmut_orig, x_temporal_by_sequence__mutmut_mutants, args, kwargs)
    return result 

temporal_by_sequence.__signature__ = _mutmut_signature(x_temporal_by_sequence__mutmut_orig)
x_temporal_by_sequence__mutmut_orig.__name__ = 'x_temporal_by_sequence'


def x_temporal_by_time__mutmut_orig(ts: Timestamp) -> TemporalQuery:
    """Create a temporal selection as-of a timestamp."""
    tq = TemporalQuery()
    tq.as_of_time.CopyFrom(ts)
    return tq


def x_temporal_by_time__mutmut_1(ts: Timestamp) -> TemporalQuery:
    """Create a temporal selection as-of a timestamp."""
    tq = None
    tq.as_of_time.CopyFrom(ts)
    return tq


def x_temporal_by_time__mutmut_2(ts: Timestamp) -> TemporalQuery:
    """Create a temporal selection as-of a timestamp."""
    tq = TemporalQuery()
    tq.as_of_time.CopyFrom(None)
    return tq

x_temporal_by_time__mutmut_mutants : ClassVar[MutantDict] = {
'x_temporal_by_time__mutmut_1': x_temporal_by_time__mutmut_1, 
    'x_temporal_by_time__mutmut_2': x_temporal_by_time__mutmut_2
}

def temporal_by_time(*args, **kwargs):
    result = _mutmut_trampoline(x_temporal_by_time__mutmut_orig, x_temporal_by_time__mutmut_mutants, args, kwargs)
    return result 

temporal_by_time.__signature__ = _mutmut_signature(x_temporal_by_time__mutmut_orig)
x_temporal_by_time__mutmut_orig.__name__ = 'x_temporal_by_time'



"""Angzarr proto definitions."""

from .types_pb2 import (
    UUID,
    Cover,
    Edition,
    DomainDivergence,
    EventPage,
    EventBook,
    Snapshot,
    CommandPage,
    CommandBook,
    CommandResponse,
    SyncCommandBook,
    SyncEventBook,
    Query,
    SequenceRange,
    SequenceSet,
    TemporalQuery,
    Projection,
    SyncMode,
    DryRunRequest,
    SagaResponse,
)
from .aggregate_pb2 import (
    SpeculateProjectorRequest,
    SpeculateSagaRequest,
    SpeculatePmRequest,
)
from .aggregate_pb2_grpc import (
    AggregateCoordinatorServiceStub,
    SpeculativeServiceStub,
)
from .query_pb2_grpc import EventQueryServiceStub
from .process_manager_pb2 import ProcessManagerHandleResponse

__all__ = [
    # Types
    "UUID",
    "Cover",
    "Edition",
    "DomainDivergence",
    "EventPage",
    "EventBook",
    "Snapshot",
    "CommandPage",
    "CommandBook",
    "CommandResponse",
    "SyncCommandBook",
    "SyncEventBook",
    "Query",
    "SequenceRange",
    "SequenceSet",
    "TemporalQuery",
    "Projection",
    "SyncMode",
    # Aggregate
    "SpeculateProjectorRequest",
    "SpeculateSagaRequest",
    "SpeculatePmRequest",
    "DryRunRequest",
    # Stubs
    "AggregateCoordinatorServiceStub",
    "SpeculativeServiceStub",
    "EventQueryServiceStub",
    # Responses
    "SagaResponse",
    "ProcessManagerHandleResponse",
]



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from angzarr_client.proto.angzarr import types_pb2 as angzarr_dot_types__pb2


class SagaServiceStub(object):
    """SagaService: client logic that coordinates across aggregates
    Two-phase protocol: Prepare (declare destinations) → Execute (with fetched state)
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetDescriptor = channel.unary_unary(
                '/angzarr.SagaService/GetDescriptor',
                request_serializer=angzarr_dot_types__pb2.GetDescriptorRequest.SerializeToString,
                response_deserializer=angzarr_dot_types__pb2.ComponentDescriptor.FromString,
                _registered_method=True)
        self.Prepare = channel.unary_unary(
                '/angzarr.SagaService/Prepare',
                request_serializer=angzarr_dot_types__pb2.SagaPrepareRequest.SerializeToString,
                response_deserializer=angzarr_dot_types__pb2.SagaPrepareResponse.FromString,
                _registered_method=True)
        self.Execute = channel.unary_unary(
                '/angzarr.SagaService/Execute',
                request_serializer=angzarr_dot_types__pb2.SagaExecuteRequest.SerializeToString,
                response_deserializer=angzarr_dot_types__pb2.SagaResponse.FromString,
                _registered_method=True)


class SagaServiceServicer(object):
    """SagaService: client logic that coordinates across aggregates
    Two-phase protocol: Prepare (declare destinations) → Execute (with fetched state)
    """

    def GetDescriptor(self, request, context):
        """Self-description: component type, subscribed domains, handled event types
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Prepare(self, request, context):
        """Phase 1: Saga declares which destination aggregates it needs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Execute(self, request, context):
        """Phase 2: Execute with source + fetched destination state
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SagaServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetDescriptor': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDescriptor,
                    request_deserializer=angzarr_dot_types__pb2.GetDescriptorRequest.FromString,
                    response_serializer=angzarr_dot_types__pb2.ComponentDescriptor.SerializeToString,
            ),
            'Prepare': grpc.unary_unary_rpc_method_handler(
                    servicer.Prepare,
                    request_deserializer=angzarr_dot_types__pb2.SagaPrepareRequest.FromString,
                    response_serializer=angzarr_dot_types__pb2.SagaPrepareResponse.SerializeToString,
            ),
            'Execute': grpc.unary_unary_rpc_method_handler(
                    servicer.Execute,
                    request_deserializer=angzarr_dot_types__pb2.SagaExecuteRequest.FromString,
                    response_serializer=angzarr_dot_types__pb2.SagaResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'angzarr.SagaService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('angzarr.SagaService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class SagaService(object):
    """SagaService: client logic that coordinates across aggregates
    Two-phase protocol: Prepare (declare destinations) → Execute (with fetched state)
    """

    @staticmethod
    def GetDescriptor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/angzarr.SagaService/GetDescriptor',
            angzarr_dot_types__pb2.GetDescriptorRequest.SerializeToString,
            angzarr_dot_types__pb2.ComponentDescriptor.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Prepare(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/angzarr.SagaService/Prepare',
            angzarr_dot_types__pb2.SagaPrepareRequest.SerializeToString,
            angzarr_dot_types__pb2.SagaPrepareResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Execute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/angzarr.SagaService/Execute',
            angzarr_dot_types__pb2.SagaExecuteRequest.SerializeToString,
            angzarr_dot_types__pb2.SagaResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)


class SagaCoordinatorServiceStub(object):
    """SagaCoordinatorService: orchestrates saga execution across domains
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ExecuteSpeculative = channel.unary_unary(
                '/angzarr.SagaCoordinatorService/ExecuteSpeculative',
                request_serializer=angzarr_dot_types__pb2.SagaExecuteRequest.SerializeToString,
                response_deserializer=angzarr_dot_types__pb2.SagaResponse.FromString,
                _registered_method=True)


class SagaCoordinatorServiceServicer(object):
    """SagaCoordinatorService: orchestrates saga execution across domains
    """

    def ExecuteSpeculative(self, request, context):
        """Speculative execution - returns commands without executing them
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SagaCoordinatorServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ExecuteSpeculative': grpc.unary_unary_rpc_method_handler(
                    servicer.ExecuteSpeculative,
                    request_deserializer=angzarr_dot_types__pb2.SagaExecuteRequest.FromString,
                    response_serializer=angzarr_dot_types__pb2.SagaResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'angzarr.SagaCoordinatorService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('angzarr.SagaCoordinatorService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class SagaCoordinatorService(object):
    """SagaCoordinatorService: orchestrates saga execution across domains
    """

    @staticmethod
    def ExecuteSpeculative(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/angzarr.SagaCoordinatorService/ExecuteSpeculative',
            angzarr_dot_types__pb2.SagaExecuteRequest.SerializeToString,
            angzarr_dot_types__pb2.SagaResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



"""Error types for the Angzarr client library."""

from typing import Optional

import grpc
from inspect import signature as _mutmut_signature
from typing import Annotated
from typing import Callable
from typing import ClassVar


MutantDict = Annotated[dict[str, Callable], "Mutant"]


def _mutmut_trampoline(orig, mutants, call_args, call_kwargs, self_arg = None):
    """Forward call to original or mutated function, depending on the environment"""
    import os
    mutant_under_test = os.environ['MUTANT_UNDER_TEST']
    if mutant_under_test == 'fail':
        from mutmut.__main__ import MutmutProgrammaticFailException
        raise MutmutProgrammaticFailException('Failed programmatically')      
    elif mutant_under_test == 'stats':
        from mutmut.__main__ import record_trampoline_hit
        record_trampoline_hit(orig.__module__ + '.' + orig.__name__)
        result = orig(*call_args, **call_kwargs)
        return result
    prefix = orig.__module__ + '.' + orig.__name__ + '__mutmut_'
    if not mutant_under_test.startswith(prefix):
        result = orig(*call_args, **call_kwargs)
        return result
    mutant_name = mutant_under_test.rpartition('.')[-1]
    if self_arg is not None:
        # call to a class method where self is not bound
        result = mutants[mutant_name](self_arg, *call_args, **call_kwargs)
    else:
        result = mutants[mutant_name](*call_args, **call_kwargs)
    return result


class ClientError(Exception):
    """Base class for client errors."""

    def xǁClientErrorǁ__init____mutmut_orig(self, message: str, cause: Optional[Exception] = None):
        super().__init__(message)
        self.message = message
        self.cause = cause

    def xǁClientErrorǁ__init____mutmut_1(self, message: str, cause: Optional[Exception] = None):
        super().__init__(None)
        self.message = message
        self.cause = cause

    def xǁClientErrorǁ__init____mutmut_2(self, message: str, cause: Optional[Exception] = None):
        super().__init__(message)
        self.message = None
        self.cause = cause

    def xǁClientErrorǁ__init____mutmut_3(self, message: str, cause: Optional[Exception] = None):
        super().__init__(message)
        self.message = message
        self.cause = None
    
    xǁClientErrorǁ__init____mutmut_mutants : ClassVar[MutantDict] = {
    'xǁClientErrorǁ__init____mutmut_1': xǁClientErrorǁ__init____mutmut_1, 
        'xǁClientErrorǁ__init____mutmut_2': xǁClientErrorǁ__init____mutmut_2, 
        'xǁClientErrorǁ__init____mutmut_3': xǁClientErrorǁ__init____mutmut_3
    }
    
    def __init__(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁClientErrorǁ__init____mutmut_orig"), object.__getattribute__(self, "xǁClientErrorǁ__init____mutmut_mutants"), args, kwargs, self)
        return result 
    
    __init__.__signature__ = _mutmut_signature(xǁClientErrorǁ__init____mutmut_orig)
    xǁClientErrorǁ__init____mutmut_orig.__name__ = 'xǁClientErrorǁ__init__'

    def __str__(self) -> str:
        if self.cause:
            return f"{self.message}: {self.cause}"
        return self.message


class ConnectionError(ClientError):
    """Failed to establish connection to the server."""

    def xǁConnectionErrorǁ__init____mutmut_orig(self, message: str):
        super().__init__(f"connection failed: {message}")

    def xǁConnectionErrorǁ__init____mutmut_1(self, message: str):
        super().__init__(None)
    
    xǁConnectionErrorǁ__init____mutmut_mutants : ClassVar[MutantDict] = {
    'xǁConnectionErrorǁ__init____mutmut_1': xǁConnectionErrorǁ__init____mutmut_1
    }
    
    def __init__(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁConnectionErrorǁ__init____mutmut_orig"), object.__getattribute__(self, "xǁConnectionErrorǁ__init____mutmut_mutants"), args, kwargs, self)
        return result 
    
    __init__.__signature__ = _mutmut_signature(xǁConnectionErrorǁ__init____mutmut_orig)
    xǁConnectionErrorǁ__init____mutmut_orig.__name__ = 'xǁConnectionErrorǁ__init__'


class TransportError(ClientError):
    """Transport-level error."""

    def xǁTransportErrorǁ__init____mutmut_orig(self, cause: Exception):
        super().__init__("transport error", cause)

    def xǁTransportErrorǁ__init____mutmut_1(self, cause: Exception):
        super().__init__(None, cause)

    def xǁTransportErrorǁ__init____mutmut_2(self, cause: Exception):
        super().__init__("transport error", None)

    def xǁTransportErrorǁ__init____mutmut_3(self, cause: Exception):
        super().__init__(cause)

    def xǁTransportErrorǁ__init____mutmut_4(self, cause: Exception):
        super().__init__("transport error", )

    def xǁTransportErrorǁ__init____mutmut_5(self, cause: Exception):
        super().__init__("XXtransport errorXX", cause)

    def xǁTransportErrorǁ__init____mutmut_6(self, cause: Exception):
        super().__init__("TRANSPORT ERROR", cause)
    
    xǁTransportErrorǁ__init____mutmut_mutants : ClassVar[MutantDict] = {
    'xǁTransportErrorǁ__init____mutmut_1': xǁTransportErrorǁ__init____mutmut_1, 
        'xǁTransportErrorǁ__init____mutmut_2': xǁTransportErrorǁ__init____mutmut_2, 
        'xǁTransportErrorǁ__init____mutmut_3': xǁTransportErrorǁ__init____mutmut_3, 
        'xǁTransportErrorǁ__init____mutmut_4': xǁTransportErrorǁ__init____mutmut_4, 
        'xǁTransportErrorǁ__init____mutmut_5': xǁTransportErrorǁ__init____mutmut_5, 
        'xǁTransportErrorǁ__init____mutmut_6': xǁTransportErrorǁ__init____mutmut_6
    }
    
    def __init__(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁTransportErrorǁ__init____mutmut_orig"), object.__getattribute__(self, "xǁTransportErrorǁ__init____mutmut_mutants"), args, kwargs, self)
        return result 
    
    __init__.__signature__ = _mutmut_signature(xǁTransportErrorǁ__init____mutmut_orig)
    xǁTransportErrorǁ__init____mutmut_orig.__name__ = 'xǁTransportErrorǁ__init__'


class GRPCError(ClientError):
    """gRPC error from the server."""

    def xǁGRPCErrorǁ__init____mutmut_orig(self, cause: grpc.RpcError):
        super().__init__("grpc error", cause)
        self._rpc_error = cause

    def xǁGRPCErrorǁ__init____mutmut_1(self, cause: grpc.RpcError):
        super().__init__(None, cause)
        self._rpc_error = cause

    def xǁGRPCErrorǁ__init____mutmut_2(self, cause: grpc.RpcError):
        super().__init__("grpc error", None)
        self._rpc_error = cause

    def xǁGRPCErrorǁ__init____mutmut_3(self, cause: grpc.RpcError):
        super().__init__(cause)
        self._rpc_error = cause

    def xǁGRPCErrorǁ__init____mutmut_4(self, cause: grpc.RpcError):
        super().__init__("grpc error", )
        self._rpc_error = cause

    def xǁGRPCErrorǁ__init____mutmut_5(self, cause: grpc.RpcError):
        super().__init__("XXgrpc errorXX", cause)
        self._rpc_error = cause

    def xǁGRPCErrorǁ__init____mutmut_6(self, cause: grpc.RpcError):
        super().__init__("GRPC ERROR", cause)
        self._rpc_error = cause

    def xǁGRPCErrorǁ__init____mutmut_7(self, cause: grpc.RpcError):
        super().__init__("grpc error", cause)
        self._rpc_error = None
    
    xǁGRPCErrorǁ__init____mutmut_mutants : ClassVar[MutantDict] = {
    'xǁGRPCErrorǁ__init____mutmut_1': xǁGRPCErrorǁ__init____mutmut_1, 
        'xǁGRPCErrorǁ__init____mutmut_2': xǁGRPCErrorǁ__init____mutmut_2, 
        'xǁGRPCErrorǁ__init____mutmut_3': xǁGRPCErrorǁ__init____mutmut_3, 
        'xǁGRPCErrorǁ__init____mutmut_4': xǁGRPCErrorǁ__init____mutmut_4, 
        'xǁGRPCErrorǁ__init____mutmut_5': xǁGRPCErrorǁ__init____mutmut_5, 
        'xǁGRPCErrorǁ__init____mutmut_6': xǁGRPCErrorǁ__init____mutmut_6, 
        'xǁGRPCErrorǁ__init____mutmut_7': xǁGRPCErrorǁ__init____mutmut_7
    }
    
    def __init__(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁGRPCErrorǁ__init____mutmut_orig"), object.__getattribute__(self, "xǁGRPCErrorǁ__init____mutmut_mutants"), args, kwargs, self)
        return result 
    
    __init__.__signature__ = _mutmut_signature(xǁGRPCErrorǁ__init____mutmut_orig)
    xǁGRPCErrorǁ__init____mutmut_orig.__name__ = 'xǁGRPCErrorǁ__init__'

    @property
    def code(self) -> grpc.StatusCode:
        """Return the gRPC status code."""
        return self._rpc_error.code()

    @property
    def details(self) -> str:
        """Return the error details."""
        return self._rpc_error.details()

    def xǁGRPCErrorǁis_not_found__mutmut_orig(self) -> bool:
        """Return True if this is a NOT_FOUND error."""
        return self.code == grpc.StatusCode.NOT_FOUND

    def xǁGRPCErrorǁis_not_found__mutmut_1(self) -> bool:
        """Return True if this is a NOT_FOUND error."""
        return self.code != grpc.StatusCode.NOT_FOUND
    
    xǁGRPCErrorǁis_not_found__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁGRPCErrorǁis_not_found__mutmut_1': xǁGRPCErrorǁis_not_found__mutmut_1
    }
    
    def is_not_found(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁGRPCErrorǁis_not_found__mutmut_orig"), object.__getattribute__(self, "xǁGRPCErrorǁis_not_found__mutmut_mutants"), args, kwargs, self)
        return result 
    
    is_not_found.__signature__ = _mutmut_signature(xǁGRPCErrorǁis_not_found__mutmut_orig)
    xǁGRPCErrorǁis_not_found__mutmut_orig.__name__ = 'xǁGRPCErrorǁis_not_found'

    def xǁGRPCErrorǁis_precondition_failed__mutmut_orig(self) -> bool:
        """Return True if this is a FAILED_PRECONDITION error."""
        return self.code == grpc.StatusCode.FAILED_PRECONDITION

    def xǁGRPCErrorǁis_precondition_failed__mutmut_1(self) -> bool:
        """Return True if this is a FAILED_PRECONDITION error."""
        return self.code != grpc.StatusCode.FAILED_PRECONDITION
    
    xǁGRPCErrorǁis_precondition_failed__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁGRPCErrorǁis_precondition_failed__mutmut_1': xǁGRPCErrorǁis_precondition_failed__mutmut_1
    }
    
    def is_precondition_failed(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁGRPCErrorǁis_precondition_failed__mutmut_orig"), object.__getattribute__(self, "xǁGRPCErrorǁis_precondition_failed__mutmut_mutants"), args, kwargs, self)
        return result 
    
    is_precondition_failed.__signature__ = _mutmut_signature(xǁGRPCErrorǁis_precondition_failed__mutmut_orig)
    xǁGRPCErrorǁis_precondition_failed__mutmut_orig.__name__ = 'xǁGRPCErrorǁis_precondition_failed'

    def xǁGRPCErrorǁis_invalid_argument__mutmut_orig(self) -> bool:
        """Return True if this is an INVALID_ARGUMENT error."""
        return self.code == grpc.StatusCode.INVALID_ARGUMENT

    def xǁGRPCErrorǁis_invalid_argument__mutmut_1(self) -> bool:
        """Return True if this is an INVALID_ARGUMENT error."""
        return self.code != grpc.StatusCode.INVALID_ARGUMENT
    
    xǁGRPCErrorǁis_invalid_argument__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁGRPCErrorǁis_invalid_argument__mutmut_1': xǁGRPCErrorǁis_invalid_argument__mutmut_1
    }
    
    def is_invalid_argument(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁGRPCErrorǁis_invalid_argument__mutmut_orig"), object.__getattribute__(self, "xǁGRPCErrorǁis_invalid_argument__mutmut_mutants"), args, kwargs, self)
        return result 
    
    is_invalid_argument.__signature__ = _mutmut_signature(xǁGRPCErrorǁis_invalid_argument__mutmut_orig)
    xǁGRPCErrorǁis_invalid_argument__mutmut_orig.__name__ = 'xǁGRPCErrorǁis_invalid_argument'


class InvalidArgumentError(ClientError):
    """Invalid argument provided by caller."""

    def xǁInvalidArgumentErrorǁ__init____mutmut_orig(self, message: str):
        super().__init__(f"invalid argument: {message}")

    def xǁInvalidArgumentErrorǁ__init____mutmut_1(self, message: str):
        super().__init__(None)
    
    xǁInvalidArgumentErrorǁ__init____mutmut_mutants : ClassVar[MutantDict] = {
    'xǁInvalidArgumentErrorǁ__init____mutmut_1': xǁInvalidArgumentErrorǁ__init____mutmut_1
    }
    
    def __init__(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁInvalidArgumentErrorǁ__init____mutmut_orig"), object.__getattribute__(self, "xǁInvalidArgumentErrorǁ__init____mutmut_mutants"), args, kwargs, self)
        return result 
    
    __init__.__signature__ = _mutmut_signature(xǁInvalidArgumentErrorǁ__init____mutmut_orig)
    xǁInvalidArgumentErrorǁ__init____mutmut_orig.__name__ = 'xǁInvalidArgumentErrorǁ__init__'


class InvalidTimestampError(ClientError):
    """Failed to parse timestamp."""

    def xǁInvalidTimestampErrorǁ__init____mutmut_orig(self, message: str):
        super().__init__(f"invalid timestamp: {message}")

    def xǁInvalidTimestampErrorǁ__init____mutmut_1(self, message: str):
        super().__init__(None)
    
    xǁInvalidTimestampErrorǁ__init____mutmut_mutants : ClassVar[MutantDict] = {
    'xǁInvalidTimestampErrorǁ__init____mutmut_1': xǁInvalidTimestampErrorǁ__init____mutmut_1
    }
    
    def __init__(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁInvalidTimestampErrorǁ__init____mutmut_orig"), object.__getattribute__(self, "xǁInvalidTimestampErrorǁ__init____mutmut_mutants"), args, kwargs, self)
        return result 
    
    __init__.__signature__ = _mutmut_signature(xǁInvalidTimestampErrorǁ__init____mutmut_orig)
    xǁInvalidTimestampErrorǁ__init____mutmut_orig.__name__ = 'xǁInvalidTimestampErrorǁ__init__'



"""Angzarr Python client library for gRPC services."""

from .client import (
    AggregateClient,
    QueryClient,
    SpeculativeClient,
    DomainClient,
    Client,
)
from .errors import (
    ClientError,
    ConnectionError,
    TransportError,
    GRPCError,
    InvalidArgumentError,
    InvalidTimestampError,
)
from .helpers import (
    domain,
    correlation_id,
    has_correlation_id,
    root_uuid,
    root_id_hex,
    edition,
    next_sequence,
    uuid_to_proto,
    proto_to_uuid,
    type_url,
    type_name_from_url,
    type_url_matches,
    now,
    parse_timestamp,
)
from .builder import CommandBuilder, QueryBuilder
from .wrappers import (
    CoverW,
    EventBookW,
    CommandBookW,
    QueryW,
    EventPageW,
    CommandPageW,
    CommandResponseW,
)

__all__ = [
    # Clients
    "AggregateClient",
    "QueryClient",
    "SpeculativeClient",
    "DomainClient",
    "Client",
    # Errors
    "ClientError",
    "ConnectionError",
    "TransportError",
    "GRPCError",
    "InvalidArgumentError",
    "InvalidTimestampError",
    # Helpers
    "domain",
    "correlation_id",
    "has_correlation_id",
    "root_uuid",
    "root_id_hex",
    "edition",
    "next_sequence",
    "uuid_to_proto",
    "proto_to_uuid",
    "type_url",
    "type_name_from_url",
    "type_url_matches",
    "now",
    "parse_timestamp",
    # Builders
    "CommandBuilder",
    "QueryBuilder",
    # Wrappers
    "CoverW",
    "EventBookW",
    "CommandBookW",
    "QueryW",
    "EventPageW",
    "CommandPageW",
    "CommandResponseW",
]



"""Client implementations for Angzarr gRPC services."""

import os
from typing import Optional

import grpc

from .proto.angzarr import (
    AggregateCoordinatorServiceStub,
    EventQueryServiceStub,
    SpeculativeServiceStub,
    CommandBook,
    CommandResponse,
    SyncCommandBook,
    DryRunRequest,
    EventBook,
    Query,
    Projection,
    SagaResponse,
    ProcessManagerHandleResponse,
    SpeculateProjectorRequest,
    SpeculateSagaRequest,
    SpeculatePmRequest,
)
from .errors import GRPCError, TransportError
from inspect import signature as _mutmut_signature
from typing import Annotated
from typing import Callable
from typing import ClassVar


MutantDict = Annotated[dict[str, Callable], "Mutant"]


def _mutmut_trampoline(orig, mutants, call_args, call_kwargs, self_arg = None):
    """Forward call to original or mutated function, depending on the environment"""
    import os
    mutant_under_test = os.environ['MUTANT_UNDER_TEST']
    if mutant_under_test == 'fail':
        from mutmut.__main__ import MutmutProgrammaticFailException
        raise MutmutProgrammaticFailException('Failed programmatically')      
    elif mutant_under_test == 'stats':
        from mutmut.__main__ import record_trampoline_hit
        record_trampoline_hit(orig.__module__ + '.' + orig.__name__)
        result = orig(*call_args, **call_kwargs)
        return result
    prefix = orig.__module__ + '.' + orig.__name__ + '__mutmut_'
    if not mutant_under_test.startswith(prefix):
        result = orig(*call_args, **call_kwargs)
        return result
    mutant_name = mutant_under_test.rpartition('.')[-1]
    if self_arg is not None:
        # call to a class method where self is not bound
        result = mutants[mutant_name](self_arg, *call_args, **call_kwargs)
    else:
        result = mutants[mutant_name](*call_args, **call_kwargs)
    return result


class QueryClient:
    """Client for the EventQueryService."""

    def xǁQueryClientǁ__init____mutmut_orig(self, channel: grpc.Channel):
        self._stub = EventQueryServiceStub(channel)
        self._channel = channel

    def xǁQueryClientǁ__init____mutmut_1(self, channel: grpc.Channel):
        self._stub = None
        self._channel = channel

    def xǁQueryClientǁ__init____mutmut_2(self, channel: grpc.Channel):
        self._stub = EventQueryServiceStub(None)
        self._channel = channel

    def xǁQueryClientǁ__init____mutmut_3(self, channel: grpc.Channel):
        self._stub = EventQueryServiceStub(channel)
        self._channel = None
    
    xǁQueryClientǁ__init____mutmut_mutants : ClassVar[MutantDict] = {
    'xǁQueryClientǁ__init____mutmut_1': xǁQueryClientǁ__init____mutmut_1, 
        'xǁQueryClientǁ__init____mutmut_2': xǁQueryClientǁ__init____mutmut_2, 
        'xǁQueryClientǁ__init____mutmut_3': xǁQueryClientǁ__init____mutmut_3
    }
    
    def __init__(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁQueryClientǁ__init____mutmut_orig"), object.__getattribute__(self, "xǁQueryClientǁ__init____mutmut_mutants"), args, kwargs, self)
        return result 
    
    __init__.__signature__ = _mutmut_signature(xǁQueryClientǁ__init____mutmut_orig)
    xǁQueryClientǁ__init____mutmut_orig.__name__ = 'xǁQueryClientǁ__init__'

    @classmethod
    def connect(cls, endpoint: str) -> "QueryClient":
        """Connect to an event query service at the given endpoint."""
        channel = grpc.insecure_channel(endpoint)
        return cls(channel)

    @classmethod
    def from_env(cls, env_var: str, default: str) -> "QueryClient":
        """Connect using an environment variable with fallback."""
        endpoint = os.environ.get(env_var, default)
        return cls.connect(endpoint)

    def xǁQueryClientǁget_event_book__mutmut_orig(self, query: Query) -> EventBook:
        """Retrieve a single EventBook for the query."""
        try:
            return self._stub.GetEventBook(query)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁQueryClientǁget_event_book__mutmut_1(self, query: Query) -> EventBook:
        """Retrieve a single EventBook for the query."""
        try:
            return self._stub.GetEventBook(None)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁQueryClientǁget_event_book__mutmut_2(self, query: Query) -> EventBook:
        """Retrieve a single EventBook for the query."""
        try:
            return self._stub.GetEventBook(query)
        except grpc.RpcError as e:
            raise GRPCError(None) from e
    
    xǁQueryClientǁget_event_book__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁQueryClientǁget_event_book__mutmut_1': xǁQueryClientǁget_event_book__mutmut_1, 
        'xǁQueryClientǁget_event_book__mutmut_2': xǁQueryClientǁget_event_book__mutmut_2
    }
    
    def get_event_book(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁQueryClientǁget_event_book__mutmut_orig"), object.__getattribute__(self, "xǁQueryClientǁget_event_book__mutmut_mutants"), args, kwargs, self)
        return result 
    
    get_event_book.__signature__ = _mutmut_signature(xǁQueryClientǁget_event_book__mutmut_orig)
    xǁQueryClientǁget_event_book__mutmut_orig.__name__ = 'xǁQueryClientǁget_event_book'

    def xǁQueryClientǁget_events__mutmut_orig(self, query: Query) -> list[EventBook]:
        """Retrieve all EventBooks matching the query."""
        try:
            return list(self._stub.GetEvents(query))
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁQueryClientǁget_events__mutmut_1(self, query: Query) -> list[EventBook]:
        """Retrieve all EventBooks matching the query."""
        try:
            return list(None)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁQueryClientǁget_events__mutmut_2(self, query: Query) -> list[EventBook]:
        """Retrieve all EventBooks matching the query."""
        try:
            return list(self._stub.GetEvents(None))
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁQueryClientǁget_events__mutmut_3(self, query: Query) -> list[EventBook]:
        """Retrieve all EventBooks matching the query."""
        try:
            return list(self._stub.GetEvents(query))
        except grpc.RpcError as e:
            raise GRPCError(None) from e
    
    xǁQueryClientǁget_events__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁQueryClientǁget_events__mutmut_1': xǁQueryClientǁget_events__mutmut_1, 
        'xǁQueryClientǁget_events__mutmut_2': xǁQueryClientǁget_events__mutmut_2, 
        'xǁQueryClientǁget_events__mutmut_3': xǁQueryClientǁget_events__mutmut_3
    }
    
    def get_events(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁQueryClientǁget_events__mutmut_orig"), object.__getattribute__(self, "xǁQueryClientǁget_events__mutmut_mutants"), args, kwargs, self)
        return result 
    
    get_events.__signature__ = _mutmut_signature(xǁQueryClientǁget_events__mutmut_orig)
    xǁQueryClientǁget_events__mutmut_orig.__name__ = 'xǁQueryClientǁget_events'

    def close(self) -> None:
        """Close the underlying channel."""
        self._channel.close()


class AggregateClient:
    """Client for the AggregateCoordinatorService."""

    def xǁAggregateClientǁ__init____mutmut_orig(self, channel: grpc.Channel):
        self._stub = AggregateCoordinatorServiceStub(channel)
        self._channel = channel

    def xǁAggregateClientǁ__init____mutmut_1(self, channel: grpc.Channel):
        self._stub = None
        self._channel = channel

    def xǁAggregateClientǁ__init____mutmut_2(self, channel: grpc.Channel):
        self._stub = AggregateCoordinatorServiceStub(None)
        self._channel = channel

    def xǁAggregateClientǁ__init____mutmut_3(self, channel: grpc.Channel):
        self._stub = AggregateCoordinatorServiceStub(channel)
        self._channel = None
    
    xǁAggregateClientǁ__init____mutmut_mutants : ClassVar[MutantDict] = {
    'xǁAggregateClientǁ__init____mutmut_1': xǁAggregateClientǁ__init____mutmut_1, 
        'xǁAggregateClientǁ__init____mutmut_2': xǁAggregateClientǁ__init____mutmut_2, 
        'xǁAggregateClientǁ__init____mutmut_3': xǁAggregateClientǁ__init____mutmut_3
    }
    
    def __init__(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁAggregateClientǁ__init____mutmut_orig"), object.__getattribute__(self, "xǁAggregateClientǁ__init____mutmut_mutants"), args, kwargs, self)
        return result 
    
    __init__.__signature__ = _mutmut_signature(xǁAggregateClientǁ__init____mutmut_orig)
    xǁAggregateClientǁ__init____mutmut_orig.__name__ = 'xǁAggregateClientǁ__init__'

    @classmethod
    def connect(cls, endpoint: str) -> "AggregateClient":
        """Connect to an aggregate coordinator at the given endpoint."""
        channel = grpc.insecure_channel(endpoint)
        return cls(channel)

    @classmethod
    def from_env(cls, env_var: str, default: str) -> "AggregateClient":
        """Connect using an environment variable with fallback."""
        endpoint = os.environ.get(env_var, default)
        return cls.connect(endpoint)

    def xǁAggregateClientǁhandle__mutmut_orig(self, command: CommandBook) -> CommandResponse:
        """Execute a command asynchronously."""
        try:
            return self._stub.Handle(command)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁAggregateClientǁhandle__mutmut_1(self, command: CommandBook) -> CommandResponse:
        """Execute a command asynchronously."""
        try:
            return self._stub.Handle(None)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁAggregateClientǁhandle__mutmut_2(self, command: CommandBook) -> CommandResponse:
        """Execute a command asynchronously."""
        try:
            return self._stub.Handle(command)
        except grpc.RpcError as e:
            raise GRPCError(None) from e
    
    xǁAggregateClientǁhandle__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁAggregateClientǁhandle__mutmut_1': xǁAggregateClientǁhandle__mutmut_1, 
        'xǁAggregateClientǁhandle__mutmut_2': xǁAggregateClientǁhandle__mutmut_2
    }
    
    def handle(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁAggregateClientǁhandle__mutmut_orig"), object.__getattribute__(self, "xǁAggregateClientǁhandle__mutmut_mutants"), args, kwargs, self)
        return result 
    
    handle.__signature__ = _mutmut_signature(xǁAggregateClientǁhandle__mutmut_orig)
    xǁAggregateClientǁhandle__mutmut_orig.__name__ = 'xǁAggregateClientǁhandle'

    def xǁAggregateClientǁhandle_sync__mutmut_orig(self, command: SyncCommandBook) -> CommandResponse:
        """Execute a command synchronously with the specified sync mode."""
        try:
            return self._stub.HandleSync(command)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁAggregateClientǁhandle_sync__mutmut_1(self, command: SyncCommandBook) -> CommandResponse:
        """Execute a command synchronously with the specified sync mode."""
        try:
            return self._stub.HandleSync(None)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁAggregateClientǁhandle_sync__mutmut_2(self, command: SyncCommandBook) -> CommandResponse:
        """Execute a command synchronously with the specified sync mode."""
        try:
            return self._stub.HandleSync(command)
        except grpc.RpcError as e:
            raise GRPCError(None) from e
    
    xǁAggregateClientǁhandle_sync__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁAggregateClientǁhandle_sync__mutmut_1': xǁAggregateClientǁhandle_sync__mutmut_1, 
        'xǁAggregateClientǁhandle_sync__mutmut_2': xǁAggregateClientǁhandle_sync__mutmut_2
    }
    
    def handle_sync(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁAggregateClientǁhandle_sync__mutmut_orig"), object.__getattribute__(self, "xǁAggregateClientǁhandle_sync__mutmut_mutants"), args, kwargs, self)
        return result 
    
    handle_sync.__signature__ = _mutmut_signature(xǁAggregateClientǁhandle_sync__mutmut_orig)
    xǁAggregateClientǁhandle_sync__mutmut_orig.__name__ = 'xǁAggregateClientǁhandle_sync'

    def xǁAggregateClientǁdry_run_handle__mutmut_orig(self, request: DryRunRequest) -> CommandResponse:
        """Execute a command in dry-run mode (no persistence)."""
        try:
            return self._stub.DryRunHandle(request)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁAggregateClientǁdry_run_handle__mutmut_1(self, request: DryRunRequest) -> CommandResponse:
        """Execute a command in dry-run mode (no persistence)."""
        try:
            return self._stub.DryRunHandle(None)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁAggregateClientǁdry_run_handle__mutmut_2(self, request: DryRunRequest) -> CommandResponse:
        """Execute a command in dry-run mode (no persistence)."""
        try:
            return self._stub.DryRunHandle(request)
        except grpc.RpcError as e:
            raise GRPCError(None) from e
    
    xǁAggregateClientǁdry_run_handle__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁAggregateClientǁdry_run_handle__mutmut_1': xǁAggregateClientǁdry_run_handle__mutmut_1, 
        'xǁAggregateClientǁdry_run_handle__mutmut_2': xǁAggregateClientǁdry_run_handle__mutmut_2
    }
    
    def dry_run_handle(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁAggregateClientǁdry_run_handle__mutmut_orig"), object.__getattribute__(self, "xǁAggregateClientǁdry_run_handle__mutmut_mutants"), args, kwargs, self)
        return result 
    
    dry_run_handle.__signature__ = _mutmut_signature(xǁAggregateClientǁdry_run_handle__mutmut_orig)
    xǁAggregateClientǁdry_run_handle__mutmut_orig.__name__ = 'xǁAggregateClientǁdry_run_handle'

    def close(self) -> None:
        """Close the underlying channel."""
        self._channel.close()


class SpeculativeClient:
    """Client for the SpeculativeService."""

    def xǁSpeculativeClientǁ__init____mutmut_orig(self, channel: grpc.Channel):
        self._stub = SpeculativeServiceStub(channel)
        self._channel = channel

    def xǁSpeculativeClientǁ__init____mutmut_1(self, channel: grpc.Channel):
        self._stub = None
        self._channel = channel

    def xǁSpeculativeClientǁ__init____mutmut_2(self, channel: grpc.Channel):
        self._stub = SpeculativeServiceStub(None)
        self._channel = channel

    def xǁSpeculativeClientǁ__init____mutmut_3(self, channel: grpc.Channel):
        self._stub = SpeculativeServiceStub(channel)
        self._channel = None
    
    xǁSpeculativeClientǁ__init____mutmut_mutants : ClassVar[MutantDict] = {
    'xǁSpeculativeClientǁ__init____mutmut_1': xǁSpeculativeClientǁ__init____mutmut_1, 
        'xǁSpeculativeClientǁ__init____mutmut_2': xǁSpeculativeClientǁ__init____mutmut_2, 
        'xǁSpeculativeClientǁ__init____mutmut_3': xǁSpeculativeClientǁ__init____mutmut_3
    }
    
    def __init__(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁSpeculativeClientǁ__init____mutmut_orig"), object.__getattribute__(self, "xǁSpeculativeClientǁ__init____mutmut_mutants"), args, kwargs, self)
        return result 
    
    __init__.__signature__ = _mutmut_signature(xǁSpeculativeClientǁ__init____mutmut_orig)
    xǁSpeculativeClientǁ__init____mutmut_orig.__name__ = 'xǁSpeculativeClientǁ__init__'

    @classmethod
    def connect(cls, endpoint: str) -> "SpeculativeClient":
        """Connect to a speculative service at the given endpoint."""
        channel = grpc.insecure_channel(endpoint)
        return cls(channel)

    @classmethod
    def from_env(cls, env_var: str, default: str) -> "SpeculativeClient":
        """Connect using an environment variable with fallback."""
        endpoint = os.environ.get(env_var, default)
        return cls.connect(endpoint)

    def xǁSpeculativeClientǁdry_run__mutmut_orig(self, request: DryRunRequest) -> CommandResponse:
        """Execute a command without persistence."""
        try:
            return self._stub.DryRunCommand(request)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁSpeculativeClientǁdry_run__mutmut_1(self, request: DryRunRequest) -> CommandResponse:
        """Execute a command without persistence."""
        try:
            return self._stub.DryRunCommand(None)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁSpeculativeClientǁdry_run__mutmut_2(self, request: DryRunRequest) -> CommandResponse:
        """Execute a command without persistence."""
        try:
            return self._stub.DryRunCommand(request)
        except grpc.RpcError as e:
            raise GRPCError(None) from e
    
    xǁSpeculativeClientǁdry_run__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁSpeculativeClientǁdry_run__mutmut_1': xǁSpeculativeClientǁdry_run__mutmut_1, 
        'xǁSpeculativeClientǁdry_run__mutmut_2': xǁSpeculativeClientǁdry_run__mutmut_2
    }
    
    def dry_run(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁSpeculativeClientǁdry_run__mutmut_orig"), object.__getattribute__(self, "xǁSpeculativeClientǁdry_run__mutmut_mutants"), args, kwargs, self)
        return result 
    
    dry_run.__signature__ = _mutmut_signature(xǁSpeculativeClientǁdry_run__mutmut_orig)
    xǁSpeculativeClientǁdry_run__mutmut_orig.__name__ = 'xǁSpeculativeClientǁdry_run'

    def xǁSpeculativeClientǁprojector__mutmut_orig(self, request: SpeculateProjectorRequest) -> Projection:
        """Speculatively execute a projector against events."""
        try:
            return self._stub.SpeculateProjector(request)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁSpeculativeClientǁprojector__mutmut_1(self, request: SpeculateProjectorRequest) -> Projection:
        """Speculatively execute a projector against events."""
        try:
            return self._stub.SpeculateProjector(None)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁSpeculativeClientǁprojector__mutmut_2(self, request: SpeculateProjectorRequest) -> Projection:
        """Speculatively execute a projector against events."""
        try:
            return self._stub.SpeculateProjector(request)
        except grpc.RpcError as e:
            raise GRPCError(None) from e
    
    xǁSpeculativeClientǁprojector__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁSpeculativeClientǁprojector__mutmut_1': xǁSpeculativeClientǁprojector__mutmut_1, 
        'xǁSpeculativeClientǁprojector__mutmut_2': xǁSpeculativeClientǁprojector__mutmut_2
    }
    
    def projector(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁSpeculativeClientǁprojector__mutmut_orig"), object.__getattribute__(self, "xǁSpeculativeClientǁprojector__mutmut_mutants"), args, kwargs, self)
        return result 
    
    projector.__signature__ = _mutmut_signature(xǁSpeculativeClientǁprojector__mutmut_orig)
    xǁSpeculativeClientǁprojector__mutmut_orig.__name__ = 'xǁSpeculativeClientǁprojector'

    def xǁSpeculativeClientǁsaga__mutmut_orig(self, request: SpeculateSagaRequest) -> SagaResponse:
        """Speculatively execute a saga against events."""
        try:
            return self._stub.SpeculateSaga(request)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁSpeculativeClientǁsaga__mutmut_1(self, request: SpeculateSagaRequest) -> SagaResponse:
        """Speculatively execute a saga against events."""
        try:
            return self._stub.SpeculateSaga(None)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁSpeculativeClientǁsaga__mutmut_2(self, request: SpeculateSagaRequest) -> SagaResponse:
        """Speculatively execute a saga against events."""
        try:
            return self._stub.SpeculateSaga(request)
        except grpc.RpcError as e:
            raise GRPCError(None) from e
    
    xǁSpeculativeClientǁsaga__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁSpeculativeClientǁsaga__mutmut_1': xǁSpeculativeClientǁsaga__mutmut_1, 
        'xǁSpeculativeClientǁsaga__mutmut_2': xǁSpeculativeClientǁsaga__mutmut_2
    }
    
    def saga(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁSpeculativeClientǁsaga__mutmut_orig"), object.__getattribute__(self, "xǁSpeculativeClientǁsaga__mutmut_mutants"), args, kwargs, self)
        return result 
    
    saga.__signature__ = _mutmut_signature(xǁSpeculativeClientǁsaga__mutmut_orig)
    xǁSpeculativeClientǁsaga__mutmut_orig.__name__ = 'xǁSpeculativeClientǁsaga'

    def xǁSpeculativeClientǁprocess_manager__mutmut_orig(
        self, request: SpeculatePmRequest
    ) -> ProcessManagerHandleResponse:
        """Speculatively execute a process manager."""
        try:
            return self._stub.SpeculateProcessManager(request)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁSpeculativeClientǁprocess_manager__mutmut_1(
        self, request: SpeculatePmRequest
    ) -> ProcessManagerHandleResponse:
        """Speculatively execute a process manager."""
        try:
            return self._stub.SpeculateProcessManager(None)
        except grpc.RpcError as e:
            raise GRPCError(e) from e

    def xǁSpeculativeClientǁprocess_manager__mutmut_2(
        self, request: SpeculatePmRequest
    ) -> ProcessManagerHandleResponse:
        """Speculatively execute a process manager."""
        try:
            return self._stub.SpeculateProcessManager(request)
        except grpc.RpcError as e:
            raise GRPCError(None) from e
    
    xǁSpeculativeClientǁprocess_manager__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁSpeculativeClientǁprocess_manager__mutmut_1': xǁSpeculativeClientǁprocess_manager__mutmut_1, 
        'xǁSpeculativeClientǁprocess_manager__mutmut_2': xǁSpeculativeClientǁprocess_manager__mutmut_2
    }
    
    def process_manager(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁSpeculativeClientǁprocess_manager__mutmut_orig"), object.__getattribute__(self, "xǁSpeculativeClientǁprocess_manager__mutmut_mutants"), args, kwargs, self)
        return result 
    
    process_manager.__signature__ = _mutmut_signature(xǁSpeculativeClientǁprocess_manager__mutmut_orig)
    xǁSpeculativeClientǁprocess_manager__mutmut_orig.__name__ = 'xǁSpeculativeClientǁprocess_manager'

    def close(self) -> None:
        """Close the underlying channel."""
        self._channel.close()


class DomainClient:
    """Combined client for aggregate and query operations on a single domain."""

    def xǁDomainClientǁ__init____mutmut_orig(self, channel: grpc.Channel):
        self.aggregate = AggregateClient(channel)
        self.query = QueryClient(channel)
        self._channel = channel

    def xǁDomainClientǁ__init____mutmut_1(self, channel: grpc.Channel):
        self.aggregate = None
        self.query = QueryClient(channel)
        self._channel = channel

    def xǁDomainClientǁ__init____mutmut_2(self, channel: grpc.Channel):
        self.aggregate = AggregateClient(None)
        self.query = QueryClient(channel)
        self._channel = channel

    def xǁDomainClientǁ__init____mutmut_3(self, channel: grpc.Channel):
        self.aggregate = AggregateClient(channel)
        self.query = None
        self._channel = channel

    def xǁDomainClientǁ__init____mutmut_4(self, channel: grpc.Channel):
        self.aggregate = AggregateClient(channel)
        self.query = QueryClient(None)
        self._channel = channel

    def xǁDomainClientǁ__init____mutmut_5(self, channel: grpc.Channel):
        self.aggregate = AggregateClient(channel)
        self.query = QueryClient(channel)
        self._channel = None
    
    xǁDomainClientǁ__init____mutmut_mutants : ClassVar[MutantDict] = {
    'xǁDomainClientǁ__init____mutmut_1': xǁDomainClientǁ__init____mutmut_1, 
        'xǁDomainClientǁ__init____mutmut_2': xǁDomainClientǁ__init____mutmut_2, 
        'xǁDomainClientǁ__init____mutmut_3': xǁDomainClientǁ__init____mutmut_3, 
        'xǁDomainClientǁ__init____mutmut_4': xǁDomainClientǁ__init____mutmut_4, 
        'xǁDomainClientǁ__init____mutmut_5': xǁDomainClientǁ__init____mutmut_5
    }
    
    def __init__(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁDomainClientǁ__init____mutmut_orig"), object.__getattribute__(self, "xǁDomainClientǁ__init____mutmut_mutants"), args, kwargs, self)
        return result 
    
    __init__.__signature__ = _mutmut_signature(xǁDomainClientǁ__init____mutmut_orig)
    xǁDomainClientǁ__init____mutmut_orig.__name__ = 'xǁDomainClientǁ__init__'

    @classmethod
    def connect(cls, endpoint: str) -> "DomainClient":
        """Connect to a domain's coordinator at the given endpoint."""
        channel = grpc.insecure_channel(endpoint)
        return cls(channel)

    @classmethod
    def from_env(cls, env_var: str, default: str) -> "DomainClient":
        """Connect using an environment variable with fallback."""
        endpoint = os.environ.get(env_var, default)
        return cls.connect(endpoint)

    def xǁDomainClientǁexecute__mutmut_orig(self, command: CommandBook) -> CommandResponse:
        """Execute a command (delegates to aggregate client)."""
        return self.aggregate.handle(command)

    def xǁDomainClientǁexecute__mutmut_1(self, command: CommandBook) -> CommandResponse:
        """Execute a command (delegates to aggregate client)."""
        return self.aggregate.handle(None)
    
    xǁDomainClientǁexecute__mutmut_mutants : ClassVar[MutantDict] = {
    'xǁDomainClientǁexecute__mutmut_1': xǁDomainClientǁexecute__mutmut_1
    }
    
    def execute(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁDomainClientǁexecute__mutmut_orig"), object.__getattribute__(self, "xǁDomainClientǁexecute__mutmut_mutants"), args, kwargs, self)
        return result 
    
    execute.__signature__ = _mutmut_signature(xǁDomainClientǁexecute__mutmut_orig)
    xǁDomainClientǁexecute__mutmut_orig.__name__ = 'xǁDomainClientǁexecute'

    def close(self) -> None:
        """Close the underlying channel."""
        self._channel.close()


class Client:
    """Combined client for aggregate, query, and speculative operations."""

    def xǁClientǁ__init____mutmut_orig(self, channel: grpc.Channel):
        self.aggregate = AggregateClient(channel)
        self.query = QueryClient(channel)
        self.speculative = SpeculativeClient(channel)
        self._channel = channel

    def xǁClientǁ__init____mutmut_1(self, channel: grpc.Channel):
        self.aggregate = None
        self.query = QueryClient(channel)
        self.speculative = SpeculativeClient(channel)
        self._channel = channel

    def xǁClientǁ__init____mutmut_2(self, channel: grpc.Channel):
        self.aggregate = AggregateClient(None)
        self.query = QueryClient(channel)
        self.speculative = SpeculativeClient(channel)
        self._channel = channel

    def xǁClientǁ__init____mutmut_3(self, channel: grpc.Channel):
        self.aggregate = AggregateClient(channel)
        self.query = None
        self.speculative = SpeculativeClient(channel)
        self._channel = channel

    def xǁClientǁ__init____mutmut_4(self, channel: grpc.Channel):
        self.aggregate = AggregateClient(channel)
        self.query = QueryClient(None)
        self.speculative = SpeculativeClient(channel)
        self._channel = channel

    def xǁClientǁ__init____mutmut_5(self, channel: grpc.Channel):
        self.aggregate = AggregateClient(channel)
        self.query = QueryClient(channel)
        self.speculative = None
        self._channel = channel

    def xǁClientǁ__init____mutmut_6(self, channel: grpc.Channel):
        self.aggregate = AggregateClient(channel)
        self.query = QueryClient(channel)
        self.speculative = SpeculativeClient(None)
        self._channel = channel

    def xǁClientǁ__init____mutmut_7(self, channel: grpc.Channel):
        self.aggregate = AggregateClient(channel)
        self.query = QueryClient(channel)
        self.speculative = SpeculativeClient(channel)
        self._channel = None
    
    xǁClientǁ__init____mutmut_mutants : ClassVar[MutantDict] = {
    'xǁClientǁ__init____mutmut_1': xǁClientǁ__init____mutmut_1, 
        'xǁClientǁ__init____mutmut_2': xǁClientǁ__init____mutmut_2, 
        'xǁClientǁ__init____mutmut_3': xǁClientǁ__init____mutmut_3, 
        'xǁClientǁ__init____mutmut_4': xǁClientǁ__init____mutmut_4, 
        'xǁClientǁ__init____mutmut_5': xǁClientǁ__init____mutmut_5, 
        'xǁClientǁ__init____mutmut_6': xǁClientǁ__init____mutmut_6, 
        'xǁClientǁ__init____mutmut_7': xǁClientǁ__init____mutmut_7
    }
    
    def __init__(self, *args, **kwargs):
        result = _mutmut_trampoline(object.__getattribute__(self, "xǁClientǁ__init____mutmut_orig"), object.__getattribute__(self, "xǁClientǁ__init____mutmut_mutants"), args, kwargs, self)
        return result 
    
    __init__.__signature__ = _mutmut_signature(xǁClientǁ__init____mutmut_orig)
    xǁClientǁ__init____mutmut_orig.__name__ = 'xǁClientǁ__init__'

    @classmethod
    def connect(cls, endpoint: str) -> "Client":
        """Connect to a server providing all services."""
        channel = grpc.insecure_channel(endpoint)
        return cls(channel)

    @classmethod
    def from_env(cls, env_var: str, default: str) -> "Client":
        """Connect using an environment variable with fallback."""
        endpoint = os.environ.get(env_var, default)
        return cls.connect(endpoint)

    def close(self) -> None:
        """Close the underlying channel."""
        self._channel.close()
